@@ -69,6 +69,8 @@ struct _RBEncoderGstPrivate {
 	gint64 total_length;
 	guint progress_id;
 	char *dest_uri;
+
+	GOutputStream *outstream;
 };
 
 G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
@@ -157,6 +159,12 @@ rb_encoder_gst_finalize (GObject *object)
 		encoder->priv->pipeline = NULL;
 	}
 
+	if (encoder->priv->outstream) {
+		g_output_stream_close (encoder->priv->outstream, NULL, NULL);
+		g_object_unref (encoder->priv->outstream);
+		encoder->priv->outstream = NULL;
+	}
+
 	g_free (encoder->priv->dest_uri);
 
         G_OBJECT_CLASS (rb_encoder_gst_parent_class)->finalize (object);
@@ -179,9 +187,6 @@ static void
 rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 {
 	GError *error = NULL;
-	guint64 dest_size;
-	GFile *file;
-	GFileInfo *file_info;
 
 	g_return_if_fail (encoder->priv->completion_emitted == FALSE);
 
@@ -203,24 +208,27 @@ rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 		g_error_free (error);
 	}
 
-	/* find the size of the output file, assuming we can get at it with gio */
-	dest_size = 0;
-	file = g_file_new_for_uri (encoder->priv->dest_uri);
-	file_info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NONE, NULL, &error);
-	if (error != NULL) {
-		rb_debug ("couldn't get size of destination %s: %s",
-			  encoder->priv->dest_uri,
-			  error->message);
-		g_clear_error (&error);
-	} else {
-		dest_size = g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
-		rb_debug ("destination file size: %" G_GUINT64_FORMAT, dest_size);
-		g_object_unref (file_info);
-	}
-	g_object_unref (file);
-
 	encoder->priv->completion_emitted = TRUE;
-	_rb_encoder_emit_completed (RB_ENCODER (encoder), dest_size);
+	_rb_encoder_emit_completed (RB_ENCODER (encoder));
+}
+
+static void
+output_close_cb (GOutputStream *stream, GAsyncResult *result, RBEncoderGst *encoder)
+{
+	GError *error = NULL;
+	rb_debug ("finished closing output stream");
+	g_output_stream_close_finish (encoder->priv->outstream, result, &error);
+	if (error != NULL) {
+		rb_debug ("error closing output stream: %s", error->message);
+		g_error_free (error);
+	}
+
+	rb_encoder_gst_emit_completed (encoder);
+
+	g_object_unref (encoder->priv->outstream);
+	encoder->priv->outstream = NULL;
+
+	g_object_unref (encoder);
 }
 
 static gboolean
@@ -252,14 +260,23 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 		break;
 
 	case GST_MESSAGE_EOS:
-		rb_debug ("received EOS");
 
 		gst_element_set_state (encoder->priv->pipeline, GST_STATE_NULL);
+		if (encoder->priv->outstream != NULL) {
+			rb_debug ("received EOS, closing output stream");
+			g_output_stream_close_async (encoder->priv->outstream,
+						     G_PRIORITY_DEFAULT,
+						     NULL,
+						     (GAsyncReadyCallback) output_close_cb,
+						     g_object_ref (encoder));
+		} else {
+			rb_debug ("received EOS, but there's no output stream");
+			rb_encoder_gst_emit_completed (encoder);
 
-		rb_encoder_gst_emit_completed (encoder);
+			g_object_unref (encoder->priv->pipeline);
+			encoder->priv->pipeline = NULL;
+		}
 
-		g_object_unref (encoder->priv->pipeline);
-		encoder->priv->pipeline = NULL;
 		break;
 
 	default:
@@ -632,6 +649,7 @@ attach_output_pipeline (RBEncoderGst *encoder,
 
 		if (stream != NULL) {
 			g_object_set (sink, "stream", stream, NULL);
+			encoder->priv->outstream = G_OUTPUT_STREAM (stream);
 		}
 	} else {
 		rb_debug ("unable to create giostreamsink, falling back to default sink for %s", dest);
@@ -647,6 +665,9 @@ attach_output_pipeline (RBEncoderGst *encoder,
 		}
 	}
 
+	/* provide a hook for setting sink properties */
+	_rb_encoder_emit_prepare_sink (RB_ENCODER (encoder), dest, G_OBJECT (sink));
+
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), sink);
 	gst_element_link (end, sink);
 
@@ -953,6 +974,17 @@ rb_encoder_gst_cancel (RBEncoder *encoder)
 	g_object_unref (priv->pipeline);
 	priv->pipeline = NULL;
 
+	if (priv->outstream != NULL) {
+		GError *error = NULL;
+		g_output_stream_close (priv->outstream, NULL, &error);
+		if (error != NULL) {
+			rb_debug ("error closing output stream: %s", error->message);
+			g_error_free (error);
+		}
+		g_object_unref (priv->outstream);
+		priv->outstream = NULL;
+	}
+
 	rb_encoder_gst_emit_completed (RB_ENCODER_GST (encoder));
 }
 
@@ -993,7 +1025,7 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 					     error->message);		/* I guess */
 
 		_rb_encoder_emit_error (encoder, error);
-		_rb_encoder_emit_completed (encoder, 0);
+		_rb_encoder_emit_completed (encoder);
 		g_error_free (error);
 		return FALSE;
 	}
