@@ -55,6 +55,7 @@ enum {
 	COMPLETED,
 	ERROR,
 	PREPARE_SOURCE,		/* this is on RBEncoderFactory */
+	PREPARE_SINK,		/* this is on RBEncoderFactory */
 	LAST_SIGNAL
 };
 
@@ -95,6 +96,26 @@ rb_encoder_factory_class_init (RBEncoderFactoryClass *klass)
 			      rb_marshal_VOID__STRING_OBJECT,
 			      G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_OBJECT);
+	/**
+	 * RBEncoderFactory::prepare-sink:
+	 * @factory: the #RBEncoderFactory instance
+	 * @uri: the URI for the sink
+	 * @sink: the sink object (a GstElement in fact)
+	 *
+	 * Emitted when creating a sink to write to the specified URI.
+	 * Plugins can use this when just creating a GStreamer element from the URI
+	 * isn't enough.  Typically this happens when there's no way to pass device
+	 * information through the URI format.
+	 */
+	signals[PREPARE_SINK] =
+		g_signal_new ("prepare-sink",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBEncoderFactoryClass, prepare_sink),
+			      NULL, NULL,
+			      rb_marshal_VOID__STRING_OBJECT,
+			      G_TYPE_NONE,
+			      2, G_TYPE_STRING, G_TYPE_OBJECT);
 }
 
 static void
@@ -129,9 +150,9 @@ rb_encoder_interface_init (RBEncoderIface *iface)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBEncoderIface, completed),
 			      NULL, NULL,
-			      rb_marshal_VOID__UINT64,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
-			      1, G_TYPE_UINT64);
+			      0);
 	/**
 	 * RBEncoder::error:
 	 * @encoder: the #RBEncoder instance
@@ -278,9 +299,9 @@ _rb_encoder_emit_progress (RBEncoder *encoder, double fraction)
 }
 
 void
-_rb_encoder_emit_completed (RBEncoder *encoder, guint64 dest_size)
+_rb_encoder_emit_completed (RBEncoder *encoder)
 {
-	g_signal_emit (encoder, signals[COMPLETED], 0, dest_size);
+	g_signal_emit (encoder, signals[COMPLETED], 0);
 }
 
 void
@@ -295,6 +316,12 @@ _rb_encoder_emit_prepare_source (RBEncoder *encoder, const char *uri, GObject *s
 	g_signal_emit (rb_encoder_factory_get (), signals[PREPARE_SOURCE], 0, uri, source);
 }
 
+void
+_rb_encoder_emit_prepare_sink (RBEncoder *encoder, const char *uri, GObject *sink)
+{
+	g_signal_emit (rb_encoder_factory_get (), signals[PREPARE_SINK], 0, uri, sink);
+}
+
 GQuark
 rb_encoder_error_quark (void)
 {
