@@ -82,7 +82,7 @@ struct _RBEncoderIface
 
 	/* signals */
 	void (*progress) (RBEncoder *encoder,  double fraction);
-	void (*completed) (RBEncoder *encoder, guint64 dest_size);
+	void (*completed) (RBEncoder *encoder);
 	void (*error) (RBEncoder *encoder, GError *error);
 };
 
@@ -92,6 +92,7 @@ struct _RBEncoderFactoryClass
 
 	/* signals */
 	void (*prepare_source) (RBEncoderFactory *factory, const char *uri, GObject *source);
+	void (*prepare_sink) (RBEncoderFactory *factory, const char *uri, GObject *sink);
 };
 
 struct _RBEncoderFactory
@@ -119,10 +120,11 @@ gboolean	rb_encoder_get_preferred_mimetype (RBEncoder *encoder,
 
 /* only to be used by subclasses */
 void	_rb_encoder_emit_progress (RBEncoder *encoder, double fraction);
-void	_rb_encoder_emit_completed (RBEncoder *encoder, guint64 dest_size);
+void	_rb_encoder_emit_completed (RBEncoder *encoder);
 void	_rb_encoder_emit_error (RBEncoder *encoder, GError *error);
 
 void	_rb_encoder_emit_prepare_source (RBEncoder *encoder, const char *uri, GObject *source);
+void	_rb_encoder_emit_prepare_sink (RBEncoder *encoder, const char *uri, GObject *sink);
 
 G_END_DECLS
 
