@@ -316,7 +316,7 @@ rb_uri_resolve_symlink (const char *uri, GError **error)
 	while (link_count < MAX_LINK_LEVEL) {
 		GFile *parent;
 		GFile *new_file;
-		char *target;
+		const char *target;
 
 		/* look for a symlink target */
 		file_info = g_file_query_info (file,
@@ -347,10 +347,8 @@ rb_uri_resolve_symlink (const char *uri, GError **error)
 			break;
 		}
 
-		target = g_file_info_get_attribute_as_string (file_info, attr);
-
+		target = g_file_info_get_attribute_byte_string (file_info, attr);
 		new_file = g_file_resolve_relative_path (parent, target);
-		g_free (target);
 		g_object_unref (parent);
 
 		g_object_unref (file_info);
@@ -1102,6 +1100,11 @@ rb_uri_create_parent_dirs (const char *uri, GError **error)
 	GError *l_error = NULL;
 #endif
 
+	/* ignore internal URI schemes */
+	if (g_str_has_prefix (uri, "xrb")) {
+		return TRUE;
+	}
+
 	file = g_file_new_for_uri (uri);
 	parent = g_file_get_parent (file);
 	g_object_unref (file);
@@ -1174,6 +1177,11 @@ rb_uri_get_filesystem_type (const char *uri)
 	char *fstype = NULL;
 	GError *error = NULL;
 
+	/* ignore our own internal URI schemes */
+	if (g_str_has_prefix (uri, "xrb")) {
+		return NULL;
+	}
+
 	/* if the file doesn't exist, walk up the directory structure
 	 * until we find something that does.
 	 */
@@ -1199,6 +1207,17 @@ rb_uri_get_filesystem_type (const char *uri)
 	return fstype;
 }
 
+/**
+ * rb_sanitize_path_for_msdos_filesystem:
+ * @path: a path to sanitize (modified in place)
+ */
+void
+rb_sanitize_path_for_msdos_filesystem (char *path)
+{
+	g_strdelimit (path, "\"", '\'');
+	g_strdelimit (path, ":|<>*?\\", '_');
+}
+
 /**
  * rb_sanitize_uri_for_filesystem:
  * @uri: a URI to sanitize
@@ -1229,8 +1248,7 @@ rb_sanitize_uri_for_filesystem (const char *uri)
 			return g_strdup (uri);
 		}
 
-		g_strdelimit (full_path, "\"", '\'');
-		g_strdelimit (full_path, ":|<>*?\\", '_');
+		rb_sanitize_path_for_msdos_filesystem (full_path);
 
 		/* create a new uri from this */
 		sane_uri = g_filename_to_uri (full_path, hostname, &error);
