@@ -10,6 +10,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -48,11 +56,14 @@ const char RB_STOCK_SET_STAR[] = "rhythmbox-set-star";
 const char RB_STOCK_UNSET_STAR[] = "rhythmbox-unset-star";
 const char RB_STOCK_NO_STAR[] = "rhythmbox-no-star";
 const char RB_STOCK_PODCAST[] = "library-podcast";
+const char RB_STOCK_PODCAST_NEW[] = "podcast-new";
 const char RB_STOCK_BROWSER[] = "stock_music-library";
 const char RB_STOCK_PLAYLIST[] = "playlist";
+const char RB_STOCK_PLAYLIST_NEW[] = "playlist-new";
 const char RB_STOCK_AUTO_PLAYLIST[] = "playlist-automatic";
-const char GNOME_MEDIA_SHUFFLE[] = "stock_shuffle";
-const char GNOME_MEDIA_REPEAT[] = "stock_repeat";
+const char RB_STOCK_AUTO_PLAYLIST_NEW[] = "playlist-automatic-new";
+const char GNOME_MEDIA_SHUFFLE[] = "media-playlist-shuffle";
+const char GNOME_MEDIA_REPEAT[] = "media-playlist-repeat";
 const char GNOME_MEDIA_EJECT[] = "media-eject";
 
 static RBInlineIconData inline_icons[] = {
@@ -69,13 +80,19 @@ rb_stock_icons_init (void)
 	int icon_size;
 	char *dot_icon_dir;
 
-	/* add our icon search paths */
+	/* add our icon search paths.  the rb_dot_dir() path is deprecated
+	 * and should be removed at some point.
+	 */
 	dot_icon_dir = g_build_filename (rb_dot_dir (), "icons", NULL);
 	gtk_icon_theme_append_search_path (theme, dot_icon_dir);
 	g_free (dot_icon_dir);
 
+	dot_icon_dir = g_build_filename (rb_user_data_dir (), "icons", NULL);
+	gtk_icon_theme_append_search_path (theme, dot_icon_dir);
+	g_free (dot_icon_dir);
+
 	gtk_icon_theme_append_search_path (theme, SHARE_DIR G_DIR_SEPARATOR_S "icons");
-#ifdef SHARE_UNINSTALLED_DIR
+#ifdef USE_UNINSTALLED_DIRS
 	gtk_icon_theme_append_search_path (theme, SHARE_UNINSTALLED_DIR G_DIR_SEPARATOR_S "icons");
 #endif
 
