@@ -6,6 +6,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -63,6 +71,55 @@ rb_metadata_dbus_get_string (DBusMessageIter *iter, gchar **value)
 	return TRUE;
 }
 
+gboolean
+rb_metadata_dbus_get_strv (DBusMessageIter *iter, char ***strv)
+{
+	guint32 count;
+	guint32 i;
+
+	/* strv is stored as a count followed by that many strings */
+	if (rb_metadata_dbus_get_uint32 (iter, &count) == FALSE) {
+		return FALSE;
+	}
+
+	if (count == 0) {
+		*strv = NULL;
+		return TRUE;
+	}
+
+	*strv = g_new0 (char *, count+1);
+	for (i = 0; i < count; i++) {
+		if (rb_metadata_dbus_get_string (iter, (*strv)+i) == FALSE) {
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
+gboolean
+rb_metadata_dbus_add_strv (DBusMessageIter *iter, char **strv)
+{
+	guint32 count;
+	guint32 i;
+
+	if (strv == NULL) {
+		count = 0;
+	} else {
+		count = g_strv_length ((char **)strv);
+	}
+
+	if (!dbus_message_iter_append_basic (iter, DBUS_TYPE_UINT32, &count)) {
+		return FALSE;
+	}
+
+	for (i=0; i < count; i++) {
+		if (!dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &strv[i])) {
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 gboolean
 rb_metadata_dbus_add_to_message (RBMetaData *md, DBusMessageIter *iter)
 {
