@@ -2,13 +2,21 @@
  *
  *  arch-tag: Implementation of audiocd source object (based of the ipod source)
  *
- *  Copyright (C) 2005-2006 James Livingston  <jrl@ids.org.au>
+ *  Copyright (C) 2005-2006 James Livingston  <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -30,28 +38,35 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <gst/gst.h>
-#include <totem-disc.h>
+#include <gst/cdda/gstcddabasesrc.h>
 
 #include "rb-plugin.h"
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
+#include "rb-shell.h"
 #include "rb-audiocd-source.h"
 #include "rb-util.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 
-#ifdef HAVE_MUSICBRAINZ
-#include "sj-metadata-musicbrainz.h"
+#ifdef HAVE_SJ_METADATA_GETTER
+#include "sj-metadata-getter.h"
 #include "sj-structures.h"
 #endif
 
+enum
+{
+	PROP_0,
+	PROP_SEARCH_TYPE
+};
+
 static void rb_audiocd_source_dispose (GObject *object);
-static GObject *rb_audiocd_source_constructor (GType type, guint n_construct_properties,
-					        GObjectConstructParam *construct_properties);
+static void rb_audiocd_source_finalize (GObject *object);
+static void rb_audiocd_source_constructed (GObject *object);
+static void impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
 static gboolean impl_show_popup (RBSource *source);
 static guint impl_want_uri (RBSource *source, const char *uri);
@@ -59,12 +74,30 @@ static gboolean impl_uri_is_source (RBSource *source, const char *uri);
 
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
-static gboolean impl_try_playlist (RBSource *source);
+
+static void impl_pack_paned (RBBrowserSource *source, GtkWidget *paned);
+
+#ifdef HAVE_SJ_METADATA_GETTER
+static void rb_audiocd_source_cmd_reload_metadata (GtkAction *action,
+						   RBAudioCdSource *source);
+#endif
 
 static gpointer rb_audiocd_load_songs (RBAudioCdSource *source);
 static void rb_audiocd_load_metadata (RBAudioCdSource *source, RhythmDB *db);
 static void rb_audiocd_load_metadata_cancel (RBAudioCdSource *source);
 
+static void metadata_gather_cb (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data, RBAudioCdSource *source);
+static GValue *album_artist_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source);
+static GValue *album_artist_sortname_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source);
+
+static gboolean update_album_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+static gboolean update_genre_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+static gboolean update_year_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+static gboolean update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source);
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+static void info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source);
+#endif
+
 typedef struct
 {
 	gchar *device_path;
@@ -74,28 +107,67 @@ typedef struct
 	GstElement *cdda;
 	GstElement *fakesink;
 
-#ifdef HAVE_MUSICBRAINZ
-	SjMetadata *metadata;
+	GtkWidget *box;
+	GtkWidget *artist_entry;
+	GtkWidget *artist_sort_entry;
+	GtkWidget *album_entry;
+	GtkWidget *year_entry;
+	GtkWidget *genre_entry;
+	GtkWidget *disc_number_entry;
+
+#ifdef HAVE_SJ_METADATA_GETTER
+	GtkActionGroup *action_group;
+	SjMetadataGetter *metadata;
+
+	GtkWidget *info_bar;
+	GtkWidget *info_bar_label;
+
+	char *submit_url;
 #endif
 } RBAudioCdSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE (RBAudioCdSource, rb_audiocd_source, RB_TYPE_REMOVABLE_MEDIA_SOURCE)
 #define AUDIOCD_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOCD_SOURCE, RBAudioCdSourcePrivate))
 
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 static AlbumDetails* multiple_album_dialog (GList *albums, RBAudioCdSource *source);
+
+static GtkActionEntry rb_audiocd_source_actions [] =
+{
+	{ "AudioCdSourceReloadMetadata", GTK_STOCK_REFRESH, N_("Reload"), NULL,
+	N_("Reload Album Information"),
+	G_CALLBACK (rb_audiocd_source_cmd_reload_metadata) },
+};
 #endif
+
+static RhythmDB *
+get_db_for_source (RBAudioCdSource *source)
+{
+	RBShell *shell;
+	RhythmDB *db;
+
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
+	g_object_unref (shell);
+
+	return db;
+}
+
+
 static void
 rb_audiocd_source_class_init (RBAudioCdSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
-	object_class->constructor = rb_audiocd_source_constructor;
+	object_class->constructed = rb_audiocd_source_constructed;
 	object_class->dispose = rb_audiocd_source_dispose;
+	object_class->finalize = rb_audiocd_source_finalize;
+	object_class->set_property = impl_set_property;
+	object_class->get_property = impl_get_property;
 
 	/* don't bother showing the browser/search bits */
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
@@ -105,16 +177,42 @@ rb_audiocd_source_class_init (RBAudioCdSourceClass *klass)
 	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
 	source_class->impl_uri_is_source = impl_uri_is_source;
-	source_class->impl_try_playlist = impl_try_playlist;
+	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_true_function;	/* shouldn't need this. */
 	source_class->impl_want_uri = impl_want_uri;
 
+	/* add the browser below the album info section */
+	browser_source_class->impl_pack_paned = impl_pack_paned;
+
+	g_object_class_override_property (object_class,
+					  PROP_SEARCH_TYPE,
+					  "search-type");
+
 	g_type_class_add_private (klass, sizeof (RBAudioCdSourcePrivate));
 }
 
 static void
 rb_audiocd_source_init (RBAudioCdSource *self)
 {
+}
 
+static void
+rb_audiocd_source_finalize (GObject *object)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (object);
+
+	g_free (priv->device_path);
+
+#ifdef HAVE_SJ_METADATA_GETTER
+	g_free (priv->submit_url);
+	priv->submit_url = NULL;
+#endif
+
+	if (priv->tracks) {
+		g_list_free (priv->tracks);
+		priv->tracks = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_audiocd_source_parent_class)->finalize (object);
 }
 
 static void
@@ -122,14 +220,18 @@ rb_audiocd_source_dispose (GObject *object)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (object);
 
-	if (priv->device_path) {
-		g_free (priv->device_path);
-		priv->device_path = NULL;
-	}
-	if (priv->tracks) {
-		g_list_free (priv->tracks);
-		priv->tracks = NULL;
+#ifdef HAVE_SJ_METADATA_GETTER
+	if (priv->action_group != NULL) {
+		GtkUIManager *uimanager;
+		g_object_get (object, "ui-manager", &uimanager, NULL);
+		gtk_ui_manager_remove_action_group (uimanager, priv->action_group);
+		g_object_unref (uimanager);
+
+		g_object_unref (priv->action_group);
+		priv->action_group = NULL;
 	}
+#endif
+
 	if (priv->pipeline) {
 		gst_object_unref (GST_OBJECT (priv->pipeline));
 		priv->pipeline = NULL;
@@ -138,59 +240,141 @@ rb_audiocd_source_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_audiocd_source_parent_class)->dispose (object);
 }
 
-static GObject *
-rb_audiocd_source_constructor (GType type,
-			       guint n_construct_properties,
-			       GObjectConstructParam *construct_properties)
+static void
+rb_audiocd_source_constructed (GObject *object)
 {
+	RBAudioCdSourcePrivate *priv;
 	RBAudioCdSource *source;
 	RBEntryView *entry_view;
+	RhythmDB *db;
+	RBPlugin *plugin;
+	char *ui_file;
 
-	source = RB_AUDIOCD_SOURCE (G_OBJECT_CLASS (rb_audiocd_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_audiocd_source_parent_class, constructed, object);
+	source = RB_AUDIOCD_SOURCE (object);
+	priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
 	g_object_set (G_OBJECT (source), "name", "Unknown Audio", NULL);
 
+#ifdef HAVE_SJ_METADATA_GETTER
+	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+							       "AudiocdActions",
+							       rb_audiocd_source_actions,
+							       G_N_ELEMENTS (rb_audiocd_source_actions),
+							       source);
+#endif
 	/* we want audio cds to sort by track# by default */
 	entry_view = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_set_sorting_order (entry_view, "Track", GTK_SORT_ASCENDING);
 
+	/* hide the 'album' column */
+	gtk_tree_view_column_set_visible (rb_entry_view_get_column (entry_view, RB_ENTRY_VIEW_COL_ALBUM), FALSE);
+
+	/* enable in-place editing for titles, artists, and genres */
+	rb_entry_view_set_column_editable (entry_view, RB_ENTRY_VIEW_COL_TITLE, TRUE);
+	rb_entry_view_set_column_editable (entry_view, RB_ENTRY_VIEW_COL_ARTIST, TRUE);
+	rb_entry_view_set_column_editable (entry_view, RB_ENTRY_VIEW_COL_GENRE, TRUE);
+
+	/* handle extra metadata requests for album artist and album artist sortname */
+	db = get_db_for_source (source);
+	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-request::" RHYTHMDB_PROP_ALBUM_ARTIST,
+				 G_CALLBACK (album_artist_metadata_request_cb), source, 0);
+	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-request::" RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME,
+				 G_CALLBACK (album_artist_sortname_metadata_request_cb), source, 0);
+	g_signal_connect_object (G_OBJECT (db), "entry-extra-metadata-gather",
+				 G_CALLBACK (metadata_gather_cb), source, 0);
+	g_object_unref (db);
+
+	/* set up the album info widgets */
+	g_object_get (source, "plugin", &plugin, NULL);
+	ui_file = rb_plugin_find_file (plugin, "album-info.ui");
+	g_object_unref (plugin);
+
+	if (ui_file == NULL) {
+		g_warning ("couldn't find album-info.ui");
+	} else {
+		RBAudioCdSourcePrivate *priv;
+		GtkWidget *table;
+		GtkBuilder *builder;
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+		GtkWidget *box;
+		char *message;
+#endif
+
+		priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+
+		builder = rb_builder_load (ui_file, NULL);
+		g_free (ui_file);
+
+		table = GTK_WIDGET (gtk_builder_get_object (builder, "album_info"));
+		g_assert (table != NULL);
+
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+		/* Info bar for non-Musicbrainz data */
+		priv->info_bar = gtk_info_bar_new_with_buttons (_("S_ubmit Album"), GTK_RESPONSE_OK,
+								_("Hide"), GTK_RESPONSE_CANCEL,
+								NULL);
+		message = g_strdup_printf ("<b>%s</b>\n%s", _("Could not find this album on MusicBrainz."),
+					   _("You can improve the MusicBrainz database by adding this album."));
+		priv->info_bar_label = gtk_label_new (NULL);
+		gtk_label_set_markup (GTK_LABEL (priv->info_bar_label), message);
+		gtk_label_set_justify (GTK_LABEL (priv->info_bar_label), GTK_JUSTIFY_LEFT);
+		g_free (message);
+		box = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->info_bar));
+		gtk_container_add (GTK_CONTAINER (box), priv->info_bar_label);
+		gtk_widget_show_all (box);
+		gtk_widget_set_no_show_all (priv->info_bar, TRUE);
+		g_signal_connect (G_OBJECT (priv->info_bar), "response",
+				  G_CALLBACK (info_bar_response_cb), source);
+		gtk_table_attach_defaults (GTK_TABLE (table), priv->info_bar, 0, 2, 0, 1);
+#endif
+
+		priv->artist_entry = GTK_WIDGET (gtk_builder_get_object (builder, "artist_entry"));
+		priv->artist_sort_entry = GTK_WIDGET (gtk_builder_get_object (builder, "artist_sort_entry"));
+		priv->album_entry = GTK_WIDGET (gtk_builder_get_object (builder, "album_entry"));
+		priv->year_entry = GTK_WIDGET (gtk_builder_get_object (builder, "year_entry"));
+		priv->genre_entry = GTK_WIDGET (gtk_builder_get_object (builder, "genre_entry"));
+		priv->disc_number_entry = GTK_WIDGET (gtk_builder_get_object (builder, "disc_number_entry"));
+
+		g_signal_connect_object (priv->album_entry, "focus-out-event", G_CALLBACK (update_album_cb), source, 0);
+		g_signal_connect_object (priv->genre_entry, "focus-out-event", G_CALLBACK (update_genre_cb), source, 0);
+		g_signal_connect_object (priv->year_entry, "focus-out-event", G_CALLBACK (update_year_cb), source, 0);
+		g_signal_connect_object (priv->disc_number_entry, "focus-out-event", G_CALLBACK (update_disc_number_cb), source, 0);
+
+		gtk_box_pack_start (GTK_BOX (priv->box), table, FALSE, FALSE, 0);
+		gtk_box_reorder_child (GTK_BOX (priv->box), table, 0);
+		g_object_unref (builder);
+	}
+
 	g_object_ref (G_OBJECT (source));
 	g_thread_create ((GThreadFunc)rb_audiocd_load_songs, source, FALSE, NULL);
-
-	return G_OBJECT (source);
 }
 
 RBRemovableMediaSource *
 rb_audiocd_source_new (RBPlugin *plugin,
 		       RBShell *shell,
-		       GnomeVFSVolume *volume)
+		       GVolume *volume)
 {
-	char *device_path;
 	GObject *source;
 	RhythmDBEntryType entry_type;
 	RhythmDB *db;
 	char *name;
 	char *path;
 
-	if (!rb_audiocd_is_volume_audiocd (volume))
-		return NULL;
+	g_object_get (shell, "db", &db, NULL);
 
-	g_object_get (G_OBJECT (shell), "db", &db, NULL);
-	path = gnome_vfs_volume_get_device_path (volume);
+	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	name = g_strdup_printf ("audiocd: %s", path);
 	entry_type = rhythmdb_entry_register_type (db, name);
-	g_object_unref (G_OBJECT (db));
 	g_free (name);
 	g_free (path);
+	g_object_unref (db);
 
 	entry_type->category = RHYTHMDB_ENTRY_NORMAL;
 	entry_type->can_sync_metadata = (RhythmDBEntryCanSyncFunc)rb_true_function;
-	/* TODO same the metadata somewhere */
+	/* TODO save the metadata somewhere */
 	entry_type->sync_metadata = (RhythmDBEntrySyncFunc)rb_null_function;
 
-	device_path = gnome_vfs_volume_get_device_path (volume);
-
 	source = g_object_new (RB_TYPE_AUDIOCD_SOURCE,
 			       "entry-type", entry_type,
 			       "volume", volume,
@@ -200,13 +384,48 @@ rb_audiocd_source_new (RBPlugin *plugin,
 			       "plugin", plugin,
 			       NULL);
 
-	g_free (device_path);
-
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
 
 	return RB_REMOVABLE_MEDIA_SOURCE (source);
 }
 
+static void
+impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	case PROP_SEARCH_TYPE:
+		/* ignored */
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	case PROP_SEARCH_TYPE:
+		g_value_set_enum (value, RB_SOURCE_SEARCH_NONE);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+impl_pack_paned (RBBrowserSource *source, GtkWidget *paned)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+
+	priv->box = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show_all (priv->box);
+	gtk_container_add (GTK_CONTAINER (source), priv->box);
+	gtk_box_pack_start (GTK_BOX (priv->box), paned, TRUE, TRUE, 0);
+}
+
 static void
 entry_set_string_prop (RhythmDB *db,
 		       RhythmDBEntry *entry,
@@ -241,6 +460,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 	RhythmDBEntry *entry;
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	char *audio_path;
+	guint64 duration;
 	GValue value = {0, };
 	gchar *str;
 	RhythmDBEntryType entry_type;
@@ -273,27 +493,17 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 			    &value);
 	g_value_unset (&value);
 
-	/* determine the duration */
-	{
-		GstFormat time_format = GST_FORMAT_TIME;
-		GstFormat track_format = gst_format_get_by_nick ("track");
-		gint64 duration;
-		gboolean result;
-                result = gst_element_seek (priv->fakesink, 1.0, track_format, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (guint64) track_number - 1, GST_SEEK_TYPE_NONE, -1);
-		if (result) {
-			result = gst_element_query_duration (priv->fakesink, &time_format, &duration) && time_format == GST_FORMAT_TIME;
-		}
-
-		if (result) {
-			g_value_init (&value, G_TYPE_ULONG);
-			g_value_set_ulong (&value, (gulong)(duration / GST_SECOND));
-			rhythmdb_entry_set (db, entry,
-					    RHYTHMDB_PROP_DURATION,
-					    &value);
-			g_value_unset (&value);
-		} else {
-			g_warning ("Failed to query cd track duration");
-		}
+	/* determine the duration
+	 * FIXME: http://bugzilla.gnome.org/show_bug.cgi?id=551011 */
+	if (gst_tag_list_get_uint64 (GST_CDDA_BASE_SRC(priv->cdda)->tracks[track_number - 1].tags, GST_TAG_DURATION, &duration)) {
+		g_value_init (&value, G_TYPE_ULONG);
+		g_value_set_ulong (&value, (gulong)(duration / GST_SECOND));
+		rhythmdb_entry_set (db, entry,
+				    RHYTHMDB_PROP_DURATION,
+				    &value);
+		g_value_unset (&value);
+	} else {
+		g_warning ("Failed to query cd track duration");
 	}
 
 	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_ARTIST, NULL);
@@ -370,22 +580,18 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 	return ok;
 }
 
-static RhythmDB *
-get_db_for_source (RBAudioCdSource *source)
+#ifdef HAVE_SJ_METADATA_GETTER
+static void
+rb_audiocd_source_cmd_reload_metadata (GtkAction *action, RBAudioCdSource *source)
 {
-	RBShell *shell;
 	RhythmDB *db;
 
-  	g_object_get (source, "shell", &shell, NULL);
-  	g_object_get (shell, "db", &db, NULL);
-  	g_object_unref (shell);
-
-        return db;
+	db = get_db_for_source (source);
+	rb_audiocd_load_metadata (source, db);
+	g_object_unref (db);
 }
 
-#ifdef HAVE_MUSICBRAINZ
-
-/**
+/*
  * Called by the Multiple Album dialog when the user hits return in
  * the list view
  */
@@ -400,7 +606,7 @@ album_row_activated (GtkTreeView *treeview,
 	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
 }
 
-/**
+/*
  * Utility function for when there are more than one albums
  * available. Borrowed from Sound Juicer.
  */
@@ -414,11 +620,11 @@ multiple_album_dialog (GList *albums, RBAudioCdSource *source)
 	AlbumDetails *album;
 	GtkTreeIter iter;
 	int response;
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *text_renderer;
 	RBPlugin *plugin;
-	char *glade_file;
+	char *builder_file;
 
 	gdk_threads_enter ();
 
@@ -426,22 +632,22 @@ multiple_album_dialog (GList *albums, RBAudioCdSource *source)
 	g_assert (plugin != NULL);
 
 	/* create dialog */
-	glade_file = rb_plugin_find_file (plugin, "multiple-album.glade");
+	builder_file = rb_plugin_find_file (plugin, "multiple-album.ui");
 	g_object_unref (plugin);
 
-	if (glade_file == NULL) {
-		g_warning ("couldn't find multiple-album.glade");
+	if (builder_file == NULL) {
+		g_warning ("couldn't find multiple-album.ui");
 		return NULL;
 	}
 
-	xml = glade_xml_new (glade_file, NULL, NULL);
-	g_free (glade_file);
+	builder = rb_builder_load (builder_file, NULL);
+	g_free (builder_file);
 
-	dialog = glade_xml_get_widget (xml, "multiple_dialog");
+	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "multiple_dialog"));
 	g_assert (dialog != NULL);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog),
 				      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source))));
-	albums_listview = glade_xml_get_widget (xml, "albums_listview");
+	albums_listview = GTK_WIDGET (gtk_builder_get_object (builder, "albums_listview"));
 
 	g_signal_connect (albums_listview, "row-activated", G_CALLBACK (album_row_activated), dialog);
 
@@ -496,12 +702,13 @@ multiple_album_dialog (GList *albums, RBAudioCdSource *source)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	gdk_threads_leave ();
+	g_object_unref (builder);
 	return album;
 }
 
 
 static void
-metadata_cb (SjMetadata *metadata,
+metadata_cb (SjMetadataGetter *metadata,
 	     GList *albums,
 	     GError *error,
 	     RBAudioCdSource *source)
@@ -517,17 +724,21 @@ metadata_cb (SjMetadata *metadata,
 	if (error != NULL) {
 		rb_debug ("Failed to load cd metadata: %s", error->message);
 		/* TODO display error to user? */
-		g_error_free (error);
 		g_object_unref (metadata);
+		priv->metadata = NULL;
 		return;
 	}
 	if (albums == NULL) {
 		rb_debug ("Musicbrainz didn't return any CD metadata, but didn't give an error");
 		g_object_unref (metadata);
+		priv->metadata = NULL;
 		return;
 	}
 	if (cd_track == NULL) {
 		/* empty cd? */
+		rb_debug ("no tracks on the CD?");
+		g_object_unref (metadata);
+		priv->metadata = NULL;
 		return;
 	}
 
@@ -536,6 +747,9 @@ metadata_cb (SjMetadata *metadata,
 	g_value_init (&true_value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&true_value, TRUE);
 
+	g_free (priv->submit_url);
+	priv->submit_url = NULL;
+
 	/* if we have multiple results, ask the user to pick one */
 	if (g_list_length (albums) > 1) {
 		album = multiple_album_dialog (albums, source);
@@ -544,7 +758,53 @@ metadata_cb (SjMetadata *metadata,
 	} else
 		album = (AlbumDetails *)albums->data;
 
+#if GTK_CHECK_VERSION(2,17,6)
+	if (album->metadata_source != SOURCE_MUSICBRAINZ) {
+		priv->submit_url = sj_metadata_getter_get_submit_url (metadata);
+		if (priv->submit_url != NULL)
+			gtk_widget_show (priv->info_bar);
+	}
+#endif
+
+	if (album->metadata_source == SOURCE_FALLBACK) {
+		rb_debug ("ignoring CD metadata from fallback source");
+		g_object_unref (metadata);
+		priv->metadata = NULL;
+		g_object_unref (db);
+		return;
+	}
+
+	if (album->artist != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (priv->artist_entry), album->artist);
+	}
+	if (album->artist_sortname != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (priv->artist_sort_entry), album->artist_sortname);
+	}
+	if (album->title != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (priv->album_entry), album->title);
+	}
+	if (album->release_date != NULL) {
+		char *year;
+		year = g_strdup_printf ("%d", g_date_get_year (album->release_date));
+		gtk_entry_set_text (GTK_ENTRY (priv->year_entry), year);
+		g_free (year);
+	}
+	if (album->disc_number != 0) {
+		char *num;
+		num = g_strdup_printf ("%d", album->disc_number);
+		gtk_entry_set_text (GTK_ENTRY (priv->disc_number_entry), num);
+		g_free (num);
+	}
+	if (album->genre != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (priv->genre_entry), album->genre);
+	}
+
 	g_object_set (G_OBJECT (source), "name", album->title, NULL);
+	rb_debug ("musicbrainz_albumid: %s", album->album_id);
+	rb_debug ("musicbrainz_albumartistid: %s", album->artist_id);
+	rb_debug ("album artist: %s", album->artist);
+	rb_debug ("disc number: %d", album->disc_number);
+	rb_debug ("genre: %s", album->genre);
 
 	while (album->tracks && cd_track) {
 		TrackDetails *track = (TrackDetails*)album->tracks->data;
@@ -553,12 +813,9 @@ metadata_cb (SjMetadata *metadata,
 
 		rb_debug ("storing metadata for %s - %s - %s", track->artist, album->title, track->title);
 
-		rb_debug ("genre: %s", album->genre);
 		rb_debug ("musicbrainz_trackid: %s", track->track_id);
 		rb_debug ("musicbrainz_artistid: %s", track->artist_id);
-		rb_debug ("musicbrainz_albumid: %s", album->album_id);
-		rb_debug ("musicbrainz_albumartistid: %s", album->artist_id);
-		rb_debug ("musicbrainz_sortname: %s", album->artist_sortname);
+		rb_debug ("artist sortname: %s", track->artist_sortname);
 
 		/* record track info in entry*/
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_TITLE, track->title);
@@ -569,13 +826,20 @@ metadata_cb (SjMetadata *metadata,
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID, track->artist_id);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID, album->album_id);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID, album->artist_id);
-		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME, album->artist_sortname);
+		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ARTIST_SORTNAME, track->artist_sortname);
 
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, track->duration);
 		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &value);
 		g_value_unset (&value);
 
+		if (album->disc_number != 0) {
+			g_value_init (&value, G_TYPE_ULONG);
+			g_value_set_ulong (&value, album->disc_number);
+			rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DISC_NUMBER, &value);
+			g_value_unset (&value);
+		}
+
 		/*album->release_date (could potentially have multiple values)*/
 		/* in current sj-structures.h, however, it does not */
 
@@ -604,6 +868,11 @@ metadata_cb (SjMetadata *metadata,
 		cd_track = g_list_next (cd_track);
 	}
 
+	/* And free the albums list, as it belongs to us, not
+	 * the metadata getter */
+	g_list_foreach (albums, (GFunc)album_details_free, NULL);
+	g_list_free (albums);
+
 	g_object_unref (metadata);
 	priv->metadata = NULL;
 
@@ -611,12 +880,14 @@ metadata_cb (SjMetadata *metadata,
 }
 
 static void
-metadata_cancelled_cb (SjMetadata *metadata,
+metadata_cancelled_cb (SjMetadataGetter *metadata,
 		       GList *albums,
 		       GError *error,
 		       gpointer old_source)
 {
 	/* NOTE: the source may have been finalised, and so should NOT be used*/
+	g_list_foreach (albums, (GFunc)album_details_free, NULL);
+	g_list_free (albums);
 	g_object_unref (metadata);
 }
 #endif
@@ -625,22 +896,22 @@ static void
 rb_audiocd_load_metadata (RBAudioCdSource *source,
 			  RhythmDB *db)
 {
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
-	priv->metadata = (SjMetadata*)sj_metadata_musicbrainz_new ();
-	sj_metadata_set_cdrom (priv->metadata, priv->device_path);
+	priv->metadata = sj_metadata_getter_new ();
+	sj_metadata_getter_set_cdrom (priv->metadata, priv->device_path);
 
 	g_signal_connect (G_OBJECT (priv->metadata), "metadata",
 			  G_CALLBACK (metadata_cb), source);
-	sj_metadata_list_albums (priv->metadata, NULL);
+	sj_metadata_getter_list_albums (priv->metadata, NULL);
 #endif
 }
 
 static void
 rb_audiocd_load_metadata_cancel (RBAudioCdSource *source)
 {
-#ifdef HAVE_MUSICBRAINZ
+#ifdef HAVE_SJ_METADATA_GETTER
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 
 	if (priv->metadata) {
@@ -657,10 +928,11 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	RhythmDB *db;
-	GnomeVFSVolume *volume;
+	GVolume *volume;
 
 	g_object_get (source, "volume", &volume, NULL);
-	priv->device_path = gnome_vfs_volume_get_device_path (volume);
+	priv->device_path = g_volume_get_identifier (volume,
+						     G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	g_object_unref (volume);
 
 	db = get_db_for_source (source);
@@ -683,6 +955,12 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 	gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdda, priv->fakesink, NULL);
 	gst_element_link (priv->cdda, priv->fakesink);
 
+	/* disable paranoia (if using cdparanoia) since we're only reading track information here.
+	 * this reduces cdparanoia's cache size, so the process is much faster.
+	 */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (source), "paranoia-mode"))
+		g_object_set (source, "paranoia-mode", 0, NULL);
+
 	if (rb_audiocd_scan_songs (source, db))
 		rb_audiocd_load_metadata (source, db);
 
@@ -699,7 +977,7 @@ impl_delete_thyself (RBSource *source)
 	RhythmDB *db;
 	RhythmDBEntryType entry_type;
 
-	rb_debug ("audio cd ejected\n");
+	rb_debug ("audio cd ejected");
 
 	/* cancel the loading of metadata */
 	rb_audiocd_load_metadata_cancel (RB_AUDIOCD_SOURCE (source));
@@ -715,36 +993,34 @@ impl_delete_thyself (RBSource *source)
 }
 
 gboolean
-rb_audiocd_is_volume_audiocd (GnomeVFSVolume *volume)
+rb_audiocd_is_mount_audiocd (GMount *mount)
 {
-	char *device_path;
-	GnomeVFSDeviceType device_type;
 	gboolean result = FALSE;
+#if GLIB_CHECK_VERSION(2,17,7)
+	char **types;
+	guint i;
+	GError *error = NULL;
 
-	device_type = gnome_vfs_volume_get_device_type (volume);
-	device_path = gnome_vfs_volume_get_device_path (volume);
-
-	if (device_path == NULL)
-		return FALSE;
-
-	/* for sometimes device_type is GNOME_VFS_DEVICE_TYPE_CDROM */
-	if (device_type == GNOME_VFS_DEVICE_TYPE_AUDIO_CD || device_type == GNOME_VFS_DEVICE_TYPE_CDROM) {
-		GError *error = NULL;
-		MediaType media_type;
-
-		media_type = totem_cd_detect_type (device_path, &error);
-		g_free (device_path);
-		if (error != NULL) {
-			rb_debug ("error while detecting cd: %s", error->message);
-			g_error_free (error);
-			return FALSE;
+	types = g_mount_guess_content_type_sync (mount, FALSE, NULL, &error);
+	if (types == NULL) {
+		rb_debug ("error guessing content type: %s", error->message);
+		g_clear_error (&error);
+	} else {
+		for (i = 0; types[i] != NULL; i++) {
+			if (g_str_equal (types[i], "x-content/audio-cdda") != FALSE) {
+				result = TRUE;
+				break;
+			}
 		}
-		rb_debug ("detecting new cd - totem cd media type=%d", media_type);
-		return (media_type == MEDIA_TYPE_CDDA);
+		g_strfreev (types);
 	}
-
-	g_free (device_path);
-
+#else
+	GFile *file;
+	
+	file = g_mount_get_root (mount);
+	result = g_file_has_uri_scheme (file, "cdda");
+	g_object_unref (file);
+#endif /* glib 2.17.7 */
 	return result;
 }
 
@@ -760,25 +1036,31 @@ impl_get_ui_actions (RBSource *source)
 {
 	GList *actions = NULL;
 
-#ifdef ENABLE_TRACK_TRANSFER
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceCopyAllTracks"));
-#endif
 	actions = g_list_prepend (actions, g_strdup ("RemovableSourceEject"));
 
+#ifdef HAVE_SJ_METADATA_GETTER
+	actions = g_list_prepend (actions, g_strdup ("AudioCdSourceReloadMetadata"));
+#endif
+
 	return actions;
 }
 
 static guint
 impl_want_uri (RBSource *source, const char *uri)
 {
-	GnomeVFSVolume *volume;
-	char *activation_uri;
+	GVolume *volume;
+	GMount *mount;
+	GFile *file;
 	int retval;
 
 	retval = 0;
 
-	if (g_str_has_prefix (uri, "cdda://") == FALSE)
+	file = g_file_new_for_uri (uri);
+	if (g_file_has_uri_scheme (file, "cdda") == FALSE) {
+		g_object_unref (file);
 		return 0;
+	}
 
 	g_object_get (G_OBJECT (source),
 		      "volume", &volume,
@@ -786,14 +1068,16 @@ impl_want_uri (RBSource *source, const char *uri)
 	if (volume == NULL)
 		return 0;
 
-	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
-	if (activation_uri == NULL)
-		return 0;
+	mount = g_volume_get_mount (volume);
+	if (mount) {
+		GFile *root;
 
-	if (strcmp (activation_uri, uri) == 0)
-		retval = 100;
-
-	g_free (activation_uri);
+		root = g_mount_get_root (mount);
+		retval = g_file_equal (root, file) ? 100 : 0;
+		g_object_unref (mount);
+		g_object_unref (root);
+	}
+	g_object_unref (file);
 
 	return retval;
 }
@@ -806,8 +1090,161 @@ impl_uri_is_source (RBSource *source, const char *uri)
 	return FALSE;
 }
 
-static gboolean
-impl_try_playlist (RBSource *source)
+static void
+metadata_gather_cb (RhythmDB *db, RhythmDBEntry *entry, RBStringValueMap *data, RBAudioCdSource *source)
 {
-	return TRUE;
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	GValue value = {0,};
+
+	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
+		return;
+	}
+
+	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_entry)) > 0) {
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_set_string (&value, gtk_entry_get_text (GTK_ENTRY (priv->artist_entry)));
+		rb_string_value_map_set (data, RHYTHMDB_PROP_ALBUM_ARTIST, &value);
+		g_value_unset (&value);
+	}
+
+	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_sort_entry)) > 0) {
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_set_string (&value, gtk_entry_get_text (GTK_ENTRY (priv->artist_sort_entry)));
+		rb_string_value_map_set (data, RHYTHMDB_PROP_ALBUM_ARTIST_SORTNAME, &value);
+		g_value_unset (&value);
+	}
 }
+
+
+static GValue *
+album_artist_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	GValue *value = NULL;
+
+	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
+		return NULL;
+	}
+
+	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_entry)) > 0) {
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, gtk_entry_get_text (GTK_ENTRY (priv->artist_entry)));
+	}
+
+	return value;
+}
+
+static GValue *
+album_artist_sortname_metadata_request_cb (RhythmDB *db, RhythmDBEntry *entry, RBAudioCdSource *source)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	GValue *value = NULL;
+
+	if (_rb_source_check_entry_type (RB_SOURCE (source), entry) == FALSE) {
+		return NULL;
+	}
+
+	if (gtk_entry_get_text_length (GTK_ENTRY (priv->artist_sort_entry)) > 0) {
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, gtk_entry_get_text (GTK_ENTRY (priv->artist_sort_entry)));
+	}
+
+	return value;
+}
+
+static void
+update_tracks (RBAudioCdSource *source, RhythmDBPropType property, GValue *value)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	RhythmDB *db;
+	GList *i;
+
+	db = get_db_for_source (source);
+
+	for (i = priv->tracks; i != NULL; i = i->next) {
+		rhythmdb_entry_set (db, i->data, property, value);
+	}
+
+	rhythmdb_commit (db);
+	g_object_unref (db);
+}
+
+static void
+update_tracks_string (RBAudioCdSource *source, RhythmDBPropType property, const char *str)
+{
+	GValue v = {0, };
+	g_value_init (&v, G_TYPE_STRING);
+	g_value_set_string (&v, str);
+	update_tracks (source, property, &v);
+	g_value_unset (&v);
+}
+
+static gboolean
+update_album_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	update_tracks_string (source, RHYTHMDB_PROP_ALBUM, gtk_entry_get_text (GTK_ENTRY (widget)));
+	return FALSE;
+}
+
+static gboolean
+update_genre_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	update_tracks_string (source, RHYTHMDB_PROP_GENRE, gtk_entry_get_text (GTK_ENTRY (widget)));
+	return FALSE;
+}
+
+static gboolean
+update_year_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	int year;
+	GDate date;
+	GValue v = {0, };
+
+	year = strtol (gtk_entry_get_text (GTK_ENTRY (widget)), NULL, 10);
+	g_date_clear (&date, 1);
+	g_date_set_dmy (&date, 1, 1, year);
+
+	g_value_init (&v, G_TYPE_ULONG);
+	g_value_set_ulong (&v, g_date_get_julian (&date));
+	update_tracks (source, RHYTHMDB_PROP_DATE, &v);
+	g_value_unset (&v);
+
+	return FALSE;
+}
+
+static gboolean
+update_disc_number_cb (GtkWidget *widget, GdkEventFocus *event, RBAudioCdSource *source)
+{
+	GValue v = {0, };
+
+	g_value_init (&v, G_TYPE_ULONG);
+	g_value_set_ulong (&v, strtoul (gtk_entry_get_text (GTK_ENTRY (widget)), NULL, 10));
+	update_tracks (source, RHYTHMDB_PROP_DISC_NUMBER, &v);
+	g_value_unset (&v);
+
+	return FALSE;
+}
+
+#if defined(HAVE_SJ_METADATA_GETTER) && GTK_CHECK_VERSION(2,17,6)
+static void
+info_bar_response_cb (GtkInfoBar *info_bar, gint response_id, RBAudioCdSource *source)
+{
+	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
+	GError *error = NULL;
+
+	g_return_if_fail (priv->submit_url != NULL);
+
+	if (response_id == GTK_RESPONSE_OK) {
+		if (!gtk_show_uri (NULL, priv->submit_url, GDK_CURRENT_TIME, &error)) {
+			rb_debug ("Could not launch submit URL %s: %s", priv->submit_url, error->message);
+			g_error_free (error);
+			return;
+		}
+	}
+
+	gtk_widget_hide (priv->info_bar);
+}
+#endif
+
