@@ -49,24 +49,22 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include <gconf/gconf-value.h>
 
 #include <totem-pl-parser.h>
 
 #include <libsoup/soup.h>
+#include <libsoup/soup-gnome.h>
 
 #include "eel-gconf-extensions.h"
 
-#include "rb-proxy-config.h"
 #include "rb-preferences.h"
 
 #include "rb-audioscrobbler.h"
 #include "rb-lastfm-source.h"
 
 #include "rhythmdb-query-model.h"
-#include "rb-glade-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
@@ -125,8 +123,7 @@ static void queue_ban_track (RBLastfmSource *source);
 
 static void rb_lastfm_source_class_init (RBLastfmSourceClass *klass);
 static void rb_lastfm_source_init (RBLastfmSource *source);
-static GObject *rb_lastfm_source_constructor (GType type, guint n_construct_properties,
-					      GObjectConstructParam *construct_properties);
+static void rb_lastfm_source_constructed (GObject *object);
 static void rb_lastfm_source_finalize (GObject *object);
 static void rb_lastfm_source_set_property (GObject *object,
 			                  guint prop_id,
@@ -199,6 +196,7 @@ static const char* const radio_options[][3] = {
 	{N_("Personal radio"), "lastfm://user/%s/personal", N_("%s's Personal Radio")},
 	{N_("Loved tracks"), "lastfm://user/%s/loved", N_("%s's Loved Tracks")},
 	{N_("Recommended tracks"), "lastfm://user/%s/recommended/100", N_("Tracks recommended to %s")},
+	{N_("Playlist"), "lastfm://user/%s/playlist", N_("%s's playlist")},
 	{NULL, NULL, NULL}
 };
 
@@ -257,7 +255,6 @@ struct RBLastfmSourcePrivate
 	const char *station_failed_reason;
 
 	SoupSession *soup_session;
-	RBProxyConfig *proxy_config;
 
 	guint emit_coverart_id;
 };
@@ -278,7 +275,6 @@ enum
 	PROP_0,
 	PROP_ENTRY_TYPE,
 	PROP_STATION_ENTRY_TYPE,
-	PROP_PROXY_CONFIG,
 	PROP_PLAY_ORDER
 };
 
@@ -311,7 +307,7 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 
 	object_class->finalize = rb_lastfm_source_finalize;
 	object_class->dispose = rb_lastfm_source_dispose;
-	object_class->constructor = rb_lastfm_source_constructor;
+	object_class->constructed = rb_lastfm_source_constructed;
 
 	object_class->set_property = rb_lastfm_source_set_property;
 	object_class->get_property = rb_lastfm_source_get_property;
@@ -345,14 +341,6 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 							     "Entry type for last.fm stations",
 							     RHYTHMDB_TYPE_ENTRY_TYPE,
 							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_property (object_class,
-					 PROP_PROXY_CONFIG,
-					 g_param_spec_object ("proxy-config",
-							      "RBProxyConfig",
-							      "RBProxyConfig object",
-							      RB_TYPE_PROXY_CONFIG,
-							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_override_property (object_class,
 					  PROP_PLAY_ORDER,
 					  "play-order");
@@ -413,11 +401,6 @@ rb_lastfm_source_dispose (GObject *object)
 		source->priv->db = NULL;
 	}
 
-	if (source->priv->proxy_config != NULL) {
-		g_object_unref (source->priv->proxy_config);
-		source->priv->proxy_config = NULL;
-	}
-
 	if (source->priv->soup_session != NULL) {
 		soup_session_abort (source->priv->soup_session);
 		g_object_unref (source->priv->soup_session);
@@ -465,12 +448,10 @@ rb_lastfm_source_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_lastfm_source_parent_class)->finalize (object);
 }
 
-static GObject *
-rb_lastfm_source_constructor (GType type, guint n_construct_properties,
-			      GObjectConstructParam *construct_properties)
+static void
+rb_lastfm_source_constructed (GObject *object)
 {
 	RBLastfmSource *source;
-	RBLastfmSourceClass *klass;
 	RBShell *shell;
 	GtkWidget *editor_vbox;
 	GtkWidget *editor_box;
@@ -480,17 +461,15 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	RhythmDBQueryModel *station_query_model;
 	int i;
 
-	klass = RB_LASTFM_SOURCE_CLASS (g_type_class_peek (RB_TYPE_LASTFM_SOURCE));
+	RB_CHAIN_GOBJECT_METHOD (rb_lastfm_source_parent_class, constructed, object);
+	source = RB_LASTFM_SOURCE (object);
 
-	source = RB_LASTFM_SOURCE (G_OBJECT_CLASS (rb_lastfm_source_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
-
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell),
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell,
 		      "db", &source->priv->db,
 		      "shell-player", &source->priv->shell_player,
 		      NULL);
-	g_object_unref (G_OBJECT (shell));
+	g_object_unref (shell);
 
 	g_signal_connect_object (source->priv->db,
 				 "entry-added",
@@ -627,8 +606,6 @@ rb_lastfm_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_set_model (source->priv->tracks, source->priv->query_model);
 	
 	g_object_set (source, "query-model", source->priv->query_model, NULL);
-
-	return G_OBJECT (source);
 }
 
 static void
@@ -646,10 +623,6 @@ rb_lastfm_source_set_property (GObject *object,
 	case PROP_STATION_ENTRY_TYPE:
 		source->priv->station_entry_type = g_value_get_boxed (value);
 		break;
-	case PROP_PROXY_CONFIG:
-		source->priv->proxy_config = g_value_get_object (value);
-		g_object_ref (G_OBJECT (source->priv->proxy_config));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -699,7 +672,6 @@ rb_lastfm_source_new (RBPlugin *plugin,
 		      RBShell  *shell)
 {
 	RBSource *source;
-	RBProxyConfig *proxy_config;
 	RhythmDBEntryType station_entry_type;
 	RhythmDBEntryType track_entry_type;
 	RhythmDB *db;
@@ -727,22 +699,18 @@ rb_lastfm_source_new (RBPlugin *plugin,
 		track_entry_type->pre_entry_destroy = destroy_track_data;
 	}
 
-	g_object_get (G_OBJECT (shell), "proxy-config", &proxy_config, NULL);
-
 	source = RB_SOURCE (g_object_new (RB_TYPE_LASTFM_SOURCE,
 					  "plugin", plugin,
 					  "name", _("Last.fm"),
 					  "shell", shell,
 					  "station-entry-type", station_entry_type,
 					  "entry-type", track_entry_type,
-					  "proxy-config", proxy_config,
 					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source, track_entry_type);
 
 	g_object_unref (db);
-	g_object_unref (proxy_config);
 	return source;
 }
 
@@ -1485,22 +1453,6 @@ http_response_cb (SoupSession *session, SoupMessage *req, gpointer user_data)
 	process_queue (source);
 }
 
-static void
-proxy_config_changed_cb (RBProxyConfig *config,
-			 RBLastfmSource *source)
-{
-	SoupURI *uri;
-
-	if (source->priv->soup_session) {
-		uri = rb_proxy_config_get_libsoup_uri (config);
-		g_object_set (G_OBJECT (source->priv->soup_session),
-					"proxy-uri", uri,
-					NULL);
-		if (uri)
-			soup_uri_free (uri);
-	}
-}
-
 static void
 process_queue (RBLastfmSource *source)
 {
@@ -1536,17 +1488,10 @@ process_queue (RBLastfmSource *source)
 	}
 
 	if (source->priv->soup_session == NULL) {
-		SoupURI *uri;
-
-		uri = rb_proxy_config_get_libsoup_uri (source->priv->proxy_config);
-		source->priv->soup_session = soup_session_async_new_with_options ("proxy-uri", uri, NULL);
-		if (uri)
-			soup_uri_free (uri);
-		
-		g_signal_connect_object (G_OBJECT (source->priv->proxy_config),
-					 "config-changed",
-					 G_CALLBACK (proxy_config_changed_cb),
-					 source, 0);
+		source->priv->soup_session =
+			soup_session_async_new_with_options (
+					SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_GNOME_FEATURES_2_26,
+					NULL);
 	}
 
 
@@ -2091,7 +2036,7 @@ xspf_entry_parsed (TotemPlParser *parser, const char *uri, GHashTable *metadata,
 	}
 
 	/* image URL and track auth ID are stored in entry type specific data */
-	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_IMAGE_URL);
+	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_IMAGE_URI);
 	if (value != NULL && (strcmp (value, LASTFM_NO_COVER_IMAGE) != 0)) {
 		track_data->image_url = g_strdup (value);
 	}
@@ -2101,7 +2046,7 @@ xspf_entry_parsed (TotemPlParser *parser, const char *uri, GHashTable *metadata,
 		track_data->track_auth = g_strdup (value);
 	}
 
-	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL);
+	value = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI);
 	if (value != NULL) {
 		track_data->download_url = g_strdup (value);
 		rb_debug ("track %s has a download url: %s", uri, track_data->download_url);
@@ -2163,6 +2108,7 @@ handle_playlist_response (RBLastfmSource *source, const char *body, RhythmDBEntr
 	result = totem_pl_parser_parse (parser, tmp_uri, FALSE);
 
 	switch (result) {
+	default:
 	case TOTEM_PL_PARSER_RESULT_UNHANDLED:
 	case TOTEM_PL_PARSER_RESULT_IGNORED:
 	case TOTEM_PL_PARSER_RESULT_ERROR:
