@@ -54,9 +54,7 @@
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/ipod/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/ipod/show_browser"
 
-static GObject *rb_ipod_source_constructor (GType type, 
-					    guint n_construct_properties,
-					    GObjectConstructParam *construct_properties);
+static void rb_ipod_source_constructed (GObject *object);
 static void rb_ipod_source_dispose (GObject *object);
 
 static char *impl_get_browser_key (RBSource *source);
@@ -72,7 +70,6 @@ static GList * impl_get_mime_types (RBRemovableMediaSource *source);
 static gboolean impl_track_added (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
 				  const char *dest,
-				  guint64 filesize,
 				  const char *mimetype);
 static char* impl_build_dest_uri (RBRemovableMediaSource *source,
 				  RhythmDBEntry *entry,
@@ -134,7 +131,7 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
-	object_class->constructor = rb_ipod_source_constructor;
+	object_class->constructed = rb_ipod_source_constructed;
 	object_class->dispose = rb_ipod_source_dispose;
 
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
@@ -187,15 +184,15 @@ rb_ipod_source_init (RBiPodSource *source)
 {	
 }
 
-static GObject *
-rb_ipod_source_constructor (GType type, guint n_construct_properties,
-			    GObjectConstructParam *construct_properties)
+static void
+rb_ipod_source_constructed (GObject *object)
 {
 	RBiPodSource *source;
 	RBEntryView *songs;
+	RhythmDB *db;
 
-	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_ipod_source_parent_class, constructed, object);
+	source = RB_IPOD_SOURCE (object);
 
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
@@ -204,14 +201,12 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 
 	rb_ipod_load_songs (source);
 
-        RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (source));
+        db = get_db_for_source (source);
         g_signal_connect_object (db,
                                  "entry-extra-metadata-notify::rb:coverArt",
                                  G_CALLBACK (rb_ipod_song_artwork_add_cb),
                                  RB_IPOD_SOURCE(source), 0);
-        g_object_unref (G_OBJECT (db));
-
-	return G_OBJECT (source);
+        g_object_unref (db);
 }
 
 static void
@@ -270,6 +265,8 @@ rb_ipod_source_new (RBPlugin *plugin,
 
 	volume = g_mount_get_volume (mount);
 	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	if (path == NULL)
+		path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UUID);
 	g_object_unref (volume);
 	
 	g_object_get (shell, "db", &db, NULL);
@@ -978,28 +975,22 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static void
-impl_move_to_trash (RBSource *asource)
+void
+rb_ipod_source_trash_entries (RBiPodSource *source, GList *entries)
 {
-	GList *sel, *tem;
-	RBEntryView *songs;
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDB *db;
-	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (asource);
-	RBiPodSource *source = RB_IPOD_SOURCE (asource);
+	GList *tem;
 
 	db = get_db_for_source (source);
-
-	songs = rb_source_get_entry_view (RB_SOURCE (asource));
-	sel = rb_entry_view_get_selected_entries (songs);
-	for (tem = sel; tem != NULL; tem = tem->next) {
+	for (tem = entries; tem != NULL; tem = tem->next) {
 		RhythmDBEntry *entry;
 		const gchar *uri;
 		gchar *file;
 		Itdb_Track *track;
 
 		entry = (RhythmDBEntry *)tem->data;
-		uri = rhythmdb_entry_get_string (entry,
-						 RHYTHMDB_PROP_LOCATION);
+		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 		track = g_hash_table_lookup (priv->entry_map, entry);
 		if (track == NULL) {
 			g_warning ("Couldn't find track on ipod! (%s)", uri);
@@ -1013,11 +1004,23 @@ impl_move_to_trash (RBSource *asource)
 			g_unlink (file);
 		g_free (file);
 		rhythmdb_entry_delete (db, entry);
-		rhythmdb_commit (db);
 	}
 
+	rhythmdb_commit (db);
 	g_object_unref (db);
+}
 
+static void
+impl_move_to_trash (RBSource *source)
+{
+	GList *sel;
+	RBEntryView *songs;
+
+	songs = rb_source_get_entry_view (source);
+	sel = rb_entry_view_get_selected_entries (songs);
+	rb_ipod_source_trash_entries (RB_IPOD_SOURCE (source), sel);
+
+	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
 	g_list_free (sel);
 }
 
@@ -1217,7 +1220,6 @@ static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
 		  const char *dest,
-		  guint64 filesize,
 		  const char *mimetype)
 {
 	RBiPodSource *isource = RB_IPOD_SOURCE (source);
@@ -1292,23 +1294,41 @@ build_ipod_dir_name (const char *mountpoint)
 	 */
 	char *dirname;
 	char *relpath;
+	char *ctrl_path, *ctrl_dir;
 	gint32 suffix;
 
-	suffix = g_random_int_range (0, 100);
+	/* Get the control directory first */
+	ctrl_path = itdb_get_control_dir (mountpoint);
+	if (ctrl_path == NULL) {
+		g_debug ("Couldn't find control directory for iPod at '%s'", mountpoint);
+		return NULL;
+	}
+	ctrl_dir = g_path_get_basename (ctrl_path);
+	if (ctrl_dir == NULL || *ctrl_dir == '.') {
+		g_free (ctrl_dir);
+		g_debug ("Couldn't find control directory for iPod at '%s' (got full path '%s'", mountpoint, ctrl_path);
+		g_free (ctrl_path);
+		return NULL;
+	}
+	g_free (ctrl_path);
+
+	suffix = g_random_int_range (0, 49);
 	dirname = g_strdup_printf ("F%02d", suffix);
-	relpath = g_build_filename (G_DIR_SEPARATOR_S, "iPod_Control",
+	relpath = g_build_filename (G_DIR_SEPARATOR_S, ctrl_dir,
 				    "Music", dirname, NULL);
 	g_free (dirname);
 
 	if (test_dir_on_ipod (mountpoint, relpath) != FALSE) {
+		g_free (ctrl_dir);
 		return relpath;
 	}
 
 	g_free (relpath);
-	dirname = g_strdup_printf ("f%02d", g_random_int_range (0, 100));
-	relpath = g_build_filename (G_DIR_SEPARATOR_S, "iPod_Control",
+	dirname = g_strdup_printf ("f%02d", suffix);
+	relpath = g_build_filename (G_DIR_SEPARATOR_S, ctrl_dir,
 				    "Music", dirname, NULL);
 	g_free (dirname);
+	g_free (ctrl_dir);
 
 	if (test_dir_on_ipod (mountpoint, relpath) != FALSE) {
 		return relpath;
@@ -1535,7 +1555,7 @@ impl_get_mime_types (RBRemovableMediaSource *source)
 {
 	GList *ret = NULL;
 
-	/* FIXME: we should really query HAL for this */
+	/* FIXME: we should really query MPID for this */
 	ret = g_list_prepend (ret, g_strdup ("audio/aac"));
 	ret = g_list_prepend (ret, g_strdup ("audio/mpeg"));
 
