@@ -9,7 +9,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -35,20 +35,18 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "rb-station-properties-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
 #include "rb-plugin.h"
+#include "rb-util.h"
 
 static void rb_station_properties_dialog_class_init (RBStationPropertiesDialogClass *klass);
 static void rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog);
-static GObject *rb_station_properties_dialog_constructor(GType type, guint n_construct_properties,
-						    GObjectConstructParam *construct_properties);
+static void rb_station_properties_dialog_constructed (GObject *object);
 static void rb_station_properties_dialog_dispose (GObject *object);
 static void rb_station_properties_dialog_finalize (GObject *object);
 static void rb_station_properties_dialog_set_property (GObject *object,
@@ -121,7 +119,7 @@ rb_station_properties_dialog_class_init (RBStationPropertiesDialogClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	object_class->constructor = rb_station_properties_dialog_constructor;
+	object_class->constructed = rb_station_properties_dialog_constructed;
 	object_class->set_property = rb_station_properties_dialog_set_property;
 	object_class->get_property = rb_station_properties_dialog_get_property;
 
@@ -154,19 +152,18 @@ rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog)
         dialog->priv = RB_STATION_PROPERTIES_DIALOG_GET_PRIVATE (dialog);
 }
 
-static GObject *
-rb_station_properties_dialog_constructor (GType type,
-					  guint n_construct_properties,
-					  GObjectConstructParam *construct_properties)
+static void
+rb_station_properties_dialog_constructed (GObject *object)
 {
 	RBStationPropertiesDialog *dialog;
-	GladeXML *xml;
-	char *gladefile;
+	GtkBuilder *builder;
+	char *builder_file;
+	AtkObject *lobj, *robj;
 
-	dialog = RB_STATION_PROPERTIES_DIALOG (G_OBJECT_CLASS (rb_station_properties_dialog_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_station_properties_dialog_parent_class, constructed, object);
+	dialog = RB_STATION_PROPERTIES_DIALOG (object);
 
-	g_signal_connect_object (G_OBJECT (dialog),
+	g_signal_connect_object (dialog,
 				 "response",
 				 G_CALLBACK (rb_station_properties_dialog_response_cb),
 				 dialog, 0);
@@ -175,40 +172,37 @@ rb_station_properties_dialog_constructor (GType type,
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 
-	gladefile = rb_plugin_find_file (dialog->priv->plugin, "station-properties.glade");
-	g_assert (gladefile != NULL);
-	xml = rb_glade_xml_new (gladefile,
-				"stationproperties",
-				dialog);
-	glade_xml_signal_autoconnect (xml);
-	g_free (gladefile);
+	builder_file = rb_plugin_find_file (dialog->priv->plugin, "station-properties.ui");
+	g_assert (builder_file != NULL);
+	builder = rb_builder_load (builder_file, dialog);
+	g_free (builder_file);
 
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-			   glade_xml_get_widget (xml, "stationproperties"));
+			   GTK_WIDGET (gtk_builder_get_object (builder, "stationproperties")));
 
 	dialog->priv->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
 							    GTK_STOCK_CLOSE,
 							    GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
-	/* get the widgets from the XML */
-	dialog->priv->title = glade_xml_get_widget (xml, "titleEntry");
-	dialog->priv->genre = glade_xml_get_widget (xml, "genreEntry");
-	dialog->priv->location = glade_xml_get_widget (xml, "locationEntry");
+	/* get the widgets from the builder */
+	dialog->priv->title = GTK_WIDGET (gtk_builder_get_object (builder, "titleEntry"));
+	dialog->priv->genre = GTK_WIDGET (gtk_builder_get_object (builder, "genreEntry"));
+	dialog->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "locationEntry"));
 
-	dialog->priv->lastplayed = glade_xml_get_widget (xml, "lastplayedLabel");
-	dialog->priv->playcount = glade_xml_get_widget (xml, "playcountLabel");
-	dialog->priv->bitrate = glade_xml_get_widget (xml, "bitrateLabel");
-	dialog->priv->playback_error = glade_xml_get_widget (xml, "errorLabel");
-	dialog->priv->playback_error_box = glade_xml_get_widget (xml, "errorBox");
+	dialog->priv->lastplayed = GTK_WIDGET (gtk_builder_get_object (builder, "lastplayedLabel"));
+	dialog->priv->playcount = GTK_WIDGET (gtk_builder_get_object (builder, "playcountLabel"));
+	dialog->priv->bitrate = GTK_WIDGET (gtk_builder_get_object (builder, "bitrateLabel"));
+	dialog->priv->playback_error = GTK_WIDGET (gtk_builder_get_object (builder, "errorLabel"));
+	dialog->priv->playback_error_box = GTK_WIDGET (gtk_builder_get_object (builder, "errorBox"));
 
-	rb_glade_boldify_label (xml, "titleLabel");
-	rb_glade_boldify_label (xml, "genreLabel");
-	rb_glade_boldify_label (xml, "locationLabel");
-	rb_glade_boldify_label (xml, "ratingLabel");
-	rb_glade_boldify_label (xml, "lastplayedDescLabel");
-	rb_glade_boldify_label (xml, "playcountDescLabel");
-	rb_glade_boldify_label (xml, "bitrateDescLabel");
+	rb_builder_boldify_label (builder, "titleLabel");
+	rb_builder_boldify_label (builder, "genreLabel");
+	rb_builder_boldify_label (builder, "locationLabel");
+	rb_builder_boldify_label (builder, "ratingLabel");
+	rb_builder_boldify_label (builder, "lastplayedDescLabel");
+	rb_builder_boldify_label (builder, "playcountDescLabel");
+	rb_builder_boldify_label (builder, "bitrateDescLabel");
 
 	g_signal_connect_object (G_OBJECT (dialog->priv->location),
 				 "changed",
@@ -220,11 +214,17 @@ rb_station_properties_dialog_constructor (GType type,
 				 "rated",
 				 G_CALLBACK (rb_station_properties_dialog_rated_cb),
 				 G_OBJECT (dialog), 0);
-	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (xml, "ratingVBox")),
+	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (builder, "ratingVBox")),
 			   dialog->priv->rating);
-	g_object_unref (xml);
 
-	return G_OBJECT (dialog);
+	/* add relationship between the rating label and the rating widget */
+	lobj = gtk_widget_get_accessible (GTK_WIDGET (gtk_builder_get_object (builder, "ratingLabel")));
+	robj = gtk_widget_get_accessible (dialog->priv->rating);
+
+	atk_object_add_relationship (lobj, ATK_RELATION_LABEL_FOR, robj);
+	atk_object_add_relationship (robj, ATK_RELATION_LABELLED_BY, lobj);
+
+	g_object_unref (builder);
 }
 
 static void
@@ -427,7 +427,7 @@ rb_station_properties_dialog_update_location (RBStationPropertiesDialog *dialog)
 	char *unescaped;
 
 	location = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION);
-	unescaped = gnome_vfs_unescape_string_for_display (location);
+	unescaped = g_uri_unescape_string (location, NULL);
 	gtk_entry_set_text (GTK_ENTRY (dialog->priv->location), unescaped);
 	g_free (unescaped);
 }
@@ -499,7 +499,7 @@ rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dial
 
 	if (dialog->priv->current_entry)
 		last_played = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LAST_PLAYED_STR);
-	gtk_label_set (GTK_LABEL (dialog->priv->lastplayed), last_played);
+	gtk_label_set_text (GTK_LABEL (dialog->priv->lastplayed), last_played);
 }
 
 static void
