@@ -26,12 +26,18 @@
 
 import gobject
 import gtk
+import sys
 
 def callback_with_gdk_lock(callback, data, args):
 	gtk.gdk.threads_enter()
-	v = callback(data, *args)
-	gtk.gdk.threads_leave()
-	return v
+	try:
+		v = callback(data, *args)
+		gtk.gdk.threads_leave()
+		return v
+	except Exception, e:
+		print "Exception caught in loader callback: %s" % str(e)
+		sys.excepthook(*sys.exc_info())
+		gtk.gdk.threads_leave()
 
 
 class GioLoader(object):
@@ -42,7 +48,7 @@ class GioLoader(object):
 		try:
 			(contents, length, etag) = file.load_contents_finish(result)
 			callback_with_gdk_lock(self.callback, contents, self.args)
-		except gio.Error, e:
+		except Exception, e:
 			# somehow check if we just got cancelled
 			callback_with_gdk_lock(self.callback, None, self.args)
 
@@ -54,7 +60,7 @@ class GioLoader(object):
 			file = gio.File(url)
 			file.load_contents_async(callback = self._contents_cb, cancellable=self._cancel)
 		except Exception, e:
-			print "error getting contents of %s: %s" % e
+			print "error getting contents of %s: %s" % (url, e)
 			callback(None, *args)
 
 	def cancel (self):
@@ -70,9 +76,13 @@ class GioChunkLoader(object):
 
 	def _callback_gdk(self, result):
 		gtk.gdk.threads_enter()
-		v = self._callback(result)
-		gtk.gdk.threads_leave()
-		return v
+		try:
+			v = self._callback(result)
+			gtk.gdk.threads_leave()
+			return v
+		except Exception, e:
+			gtk.gdk.threads_leave()
+			raise e
 
 	def _error_idle_cb(self, error):
 		self._callback_gdk(error)
@@ -131,7 +141,7 @@ class GioChunkLoader(object):
 			if want_size:
 				file.query_info_async(self._info_cb, gio.FILE_ATTRIBUTE_STANDARD_SIZE, cancellable=self._cancel)
 			else:
-				file.read_async(self._open_cb, cancellable=self.cancel)
+				file.read_async(self._open_cb, cancellable=self._cancel)
 		except gio.Error, e:
 			print "error reading file %s: %s" % (uri, e.message)
 			self._callback(e)
@@ -220,9 +230,13 @@ class GnomeVFSChunkLoader (object):
 
 	def _callback_gdk(self, result):
 		gtk.gdk.threads_enter()
-		v = self._callback(result)
-		gtk.gdk.threads_leave()
-		return v
+		try:
+			v = self._callback(result)
+			gtk.gdk.threads_leave()
+			return v
+		except Exception, e:
+			gtk.gdk.threads_leave()
+			raise e
 
 	def _read_cb (self, handle, buffer, exc_type, bytes_requested):
 		if exc_type:
