@@ -141,8 +141,7 @@ static guint rb_podcast_manager_signals[LAST_SIGNAL] = { 0 };
 /* functions */
 static void rb_podcast_manager_class_init 		(RBPodcastManagerClass *klass);
 static void rb_podcast_manager_init 			(RBPodcastManager *dp);
-static GObject *rb_podcast_manager_constructor 		(GType type, guint n_construct_properties,
-			   				 GObjectConstructParam *construct_properties);
+static void rb_podcast_manager_constructed		(GObject *object);
 static void rb_podcast_manager_dispose 			(GObject *object);
 static void rb_podcast_manager_finalize 		(GObject *object);
 static void rb_podcast_manager_set_property 		(GObject *object,
@@ -195,7 +194,7 @@ rb_podcast_manager_class_init (RBPodcastManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->constructor = rb_podcast_manager_constructor;
+	object_class->constructed = rb_podcast_manager_constructed;
 	object_class->dispose = rb_podcast_manager_dispose;
 	object_class->finalize = rb_podcast_manager_finalize;
 
@@ -291,21 +290,14 @@ rb_podcast_manager_init (RBPodcastManager *pd)
 	eel_gconf_monitor_add (CONF_STATE_PODCAST_PREFIX);
 }
 
-static GObject *
-rb_podcast_manager_constructor (GType type, guint n_construct_properties,
-				GObjectConstructParam *construct_properties)
+static void
+rb_podcast_manager_constructed (GObject *object)
 {
-	RBPodcastManager *pd;
-
-	pd = RB_PODCAST_MANAGER (G_OBJECT_CLASS (rb_podcast_manager_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
-
+	RBPodcastManager *pd = RB_PODCAST_MANAGER (object);
+	RB_CHAIN_GOBJECT_METHOD (rb_podcast_manager_parent_class, constructed, object);
 	pd->priv->update_interval_notify_id = eel_gconf_notification_add (CONF_STATE_PODCAST_DOWNLOAD_INTERVAL,
 	                    			       			  rb_podcast_manager_config_changed,
 	                            		       			  pd);
-
-	return G_OBJECT (pd);
-
 }
 
 static void
@@ -788,6 +780,8 @@ download_file_info_cb (GFile *source,
 	feed_folder = g_uri_escape_string (rhythmdb_entry_get_string (data->entry, RHYTHMDB_PROP_ALBUM),
 					   G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
 					   TRUE);
+	g_strdelimit (feed_folder, "/", '_');
+	g_strdelimit (esc_local_file_name, "/", '_');
 
 	/* construct local filename */
 	conf_dir_uri = rb_podcast_manager_get_podcast_dir (data->pd);
@@ -1805,6 +1799,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	GValue error_val = { 0, };
 	gulong last_post = 0;
 	gulong new_last_post;
+	const char *title;
 	GList *download_entries = NULL;
 	gboolean new_feed, updated, download_last;
 	RhythmDB *db = pd->priv->db;
@@ -1860,8 +1855,10 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	g_value_init (&title_val, G_TYPE_STRING);
 	if (data->title == NULL || strlen ((gchar *)data->title) == 0) {
 		g_value_set_string (&title_val, (gchar *) data->url);
+		title = data->url;
 	} else {
 		g_value_set_string (&title_val, (gchar *) data->title);
+		title = data->title;
 	}
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
 	g_value_unset (&title_val);
@@ -1944,7 +1941,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 
 			post_entry =
 				rb_podcast_manager_add_post (db,
-							     (gchar *) data->title,
+							     title,
 							     (gchar *) item->title,
 							     (gchar *) data->url,
 							     (gchar *) (item->author ? item->author : data->author),
