@@ -47,14 +47,13 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "rb-podcast-source.h"
 
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-statusbar.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
@@ -78,9 +77,7 @@ static void rb_podcast_source_class_init 		(RBPodcastSourceClass *klass);
 
 static void rb_podcast_source_init 			(RBPodcastSource *source);
 
-static GObject *rb_podcast_source_constructor 		(GType type,
-							 guint n_construct_properties,
-					      		 GObjectConstructParam *construct_properties);
+static void rb_podcast_source_constructed 		(GObject *object);
 
 static void rb_podcast_source_dispose 			(GObject *object);
 
@@ -280,7 +277,6 @@ struct RBPodcastSourcePrivate
 {
 	RhythmDB *db;
 
-	guint toolbar_ui_merge_id;
 	guint prefs_notify_id;
 
 	GtkWidget *vbox;
@@ -297,13 +293,9 @@ struct RBPodcastSourcePrivate
 	RhythmDBPropType search_prop;
 	RBSourceSearch *default_search;
 
-	gboolean initialized;
-
 	RBPodcastManager *podcast_mgr;
 
 	GdkPixbuf *error_pixbuf;
-
-	gboolean dispose_has_run;
 };
 
 #define RB_PODCAST_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PODCAST_SOURCE, RBPodcastSourcePrivate))
@@ -364,7 +356,7 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 
 	object_class->dispose = rb_podcast_source_dispose;
 	object_class->finalize = rb_podcast_source_finalize;
-	object_class->constructor = rb_podcast_source_constructor;
+	object_class->constructed = rb_podcast_source_constructed;
 
 	object_class->set_property = rb_podcast_source_set_property;
 	object_class->get_property = rb_podcast_source_get_property;
@@ -444,13 +436,6 @@ rb_podcast_source_dispose (GObject *object)
 	rb_debug ("dispose podcast_source");
 	source = RB_PODCAST_SOURCE (object);
 
-	if (source->priv->dispose_has_run) {
-		/* If dispose did already run, return. */
-		return;
-	}
-	/* Make sure dispose does not run twice. */
-	source->priv->dispose_has_run = TRUE;
-
 	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
@@ -476,7 +461,10 @@ rb_podcast_source_dispose (GObject *object)
 		source->priv->error_pixbuf = NULL;
 	}
 
-	eel_gconf_notification_remove (source->priv->prefs_notify_id);
+	if (source->priv->prefs_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->prefs_notify_id);
+		source->priv->prefs_notify_id = 0;
+	}
 
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->dispose (object);
 }
@@ -503,13 +491,10 @@ rb_podcast_source_finalize (GObject *object)
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->finalize (object);
 }
 
-static GObject *
-rb_podcast_source_constructor (GType type,
-			       guint n_construct_properties,
-			       GObjectConstructParam *construct_properties)
+static void
+rb_podcast_source_constructed (GObject *object)
 {
 	RBPodcastSource *source;
-	RBPodcastSourceClass *klass;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	RBShell *shell;
@@ -517,10 +502,8 @@ rb_podcast_source_constructor (GType type,
 	GPtrArray *query;
 	GtkAction *action;
 
-	klass = RB_PODCAST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PODCAST_SOURCE));
-
-	source = RB_PODCAST_SOURCE (G_OBJECT_CLASS (rb_podcast_source_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_podcast_source_parent_class, constructed, object);
+	source = RB_PODCAST_SOURCE (object);
 
 	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &source->priv->db, NULL);
@@ -795,8 +778,6 @@ rb_podcast_source_constructor (GType type,
 	rb_podcast_source_state_prefs_sync (source);
 
 	rb_podcast_source_do_query (source);
-
-	return G_OBJECT (source);
 }
 
 static void
@@ -1220,18 +1201,18 @@ static GtkWidget *
 impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
+	GtkBuilder *builder;
 	GtkWidget *cb_update_interval;
 	GtkWidget *btn_file;
 	char *download_dir;
-	GladeXML *xml;
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
 
-	xml = rb_glade_xml_new ("podcast-prefs.glade", "podcast_vbox", source);
-	source->priv->config_widget = glade_xml_get_widget (xml, "podcast_vbox");
+	builder = rb_builder_load ("podcast-prefs.ui", source);
+	source->priv->config_widget = GTK_WIDGET (gtk_builder_get_object (builder, "podcast_vbox"));
 
-	btn_file = glade_xml_get_widget (xml, "location_chooser");
+	btn_file = GTK_WIDGET (gtk_builder_get_object (builder, "location_chooser"));
 	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (btn_file),
 					      rb_music_dir (),
 					      NULL);
@@ -1245,7 +1226,7 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 			  G_CALLBACK (rb_podcast_source_btn_file_change_cb),
 			  CONF_STATE_PODCAST_DOWNLOAD_DIR);
 
-	cb_update_interval = glade_xml_get_widget (xml, "cb_update_interval");
+	cb_update_interval = GTK_WIDGET (gtk_builder_get_object (builder, "cb_update_interval"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (cb_update_interval),
 				  eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_INTERVAL));
 	g_signal_connect (cb_update_interval,
@@ -1742,11 +1723,13 @@ rb_podcast_source_start_download_cb (RBPodcastManager *pd,
 				     RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("Downloading podcast"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Start download");
-	rb_shell_hidden_notify (shell, 4000, _("Downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 }
 
 static void
@@ -1755,11 +1738,13 @@ rb_podcast_source_finish_download_cb (RBPodcastManager *pd,
 				      RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("Finished downloading podcast"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Finish download");
-	rb_shell_hidden_notify (shell, 4000, _("Finished downloading podcast"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 }
 
 static void
@@ -1768,11 +1753,13 @@ rb_podcast_source_feed_updates_available_cb (RBPodcastManager *pd,
 					     RBPodcastSource *source)
 {
 	RBShell *shell = rb_podcast_source_get_shell (source);
-	const gchar *podcast_name = rhythmdb_entry_get_string(entry, RHYTHMDB_PROP_TITLE);
+	char *podcast_name;
+
+	podcast_name = g_markup_escape_text (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE), -1);
+	rb_shell_notify_custom (shell, 4000, _("New updates available from"), podcast_name, NULL, FALSE);
 
-	rb_debug ("Updates available");
-	rb_shell_hidden_notify (shell, 4000, _("New updates available from"), NULL, podcast_name, FALSE);
 	g_object_unref (shell);
+	g_free (podcast_name);
 
 }
 
