@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -34,11 +34,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkiconfactory.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <gtk/gtk.h>
 
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
@@ -51,9 +47,7 @@
 #include "rb-util.h"
 #include "rb-file-helpers.h"
 
-static GObject *rb_removable_media_source_constructor (GType type,
-						       guint n_construct_properties,
-						       GObjectConstructParam *construct_properties);
+static void rb_removable_media_source_constructed (GObject *object);
 static void rb_removable_media_source_dispose (GObject *object);
 
 static void rb_removable_media_source_set_property (GObject *object,
@@ -66,16 +60,17 @@ static void rb_removable_media_source_get_property (GObject *object,
 			                  GParamSpec *pspec);
 
 static void impl_delete_thyself (RBSource *source);
-#ifdef ENABLE_TRACK_TRANSFER
 static void impl_paste (RBSource *source, GList *entries);
-#endif
 static gboolean impl_receive_drag (RBSource *asource, GtkSelectionData *data);
+static gboolean impl_should_paste (RBRemovableMediaSource *source,
+				   RhythmDBEntry *entry);
 static guint impl_want_uri (RBSource *source, const char *uri);
 static gboolean impl_uri_is_source (RBSource *source, const char *uri);
 
 typedef struct
 {
-	GnomeVFSVolume *volume;
+	GVolume *volume;
+	GMount *mount;
 } RBRemovableMediaSourcePrivate;
 
 G_DEFINE_TYPE (RBRemovableMediaSource, rb_removable_media_source, RB_TYPE_BROWSER_SOURCE)
@@ -85,6 +80,7 @@ enum
 {
 	PROP_0,
 	PROP_VOLUME,
+	PROP_MOUNT,
 };
 
 static void
@@ -94,7 +90,7 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
-	object_class->constructor = rb_removable_media_source_constructor;
+	object_class->constructed = rb_removable_media_source_constructed;
 	object_class->dispose = rb_removable_media_source_dispose;
 	object_class->set_property = rb_removable_media_source_set_property;
 	object_class->get_property = rb_removable_media_source_get_property;
@@ -104,9 +100,7 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
-#ifdef ENABLE_TRACK_TRANSFER
   	source_class->impl_paste = impl_paste;
-#endif
   	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_delete = NULL;
@@ -118,12 +112,21 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 	browser_source_class->impl_get_paned_key = NULL;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
+	klass->impl_should_paste = impl_should_paste;
+
 	g_object_class_install_property (object_class,
 					 PROP_VOLUME,
 					 g_param_spec_object ("volume",
 							      "Volume",
-							      "GnomeVfs Volume",
-							      GNOME_VFS_TYPE_VOLUME,
+							      "GIO Volume",
+							      G_TYPE_VOLUME,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_MOUNT,
+					 g_param_spec_object ("mount",
+							      "Mount",
+							      "GIO Mount",
+							      G_TYPE_MOUNT,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (klass, sizeof (RBRemovableMediaSourcePrivate));
@@ -134,52 +137,76 @@ rb_removable_media_source_init (RBRemovableMediaSource *self)
 {
 }
 
-static GObject *
-rb_removable_media_source_constructor (GType type, guint n_construct_properties,
-				       GObjectConstructParam *construct_properties)
+static void
+rb_removable_media_source_constructed (GObject *object)
 {
-	GObject *source;
-	GnomeVFSVolume *volume;
-	GnomeVFSDrive *drive;
+	GMount *mount;
+	GIcon *icon = NULL;
 	char *display_name;
-	gint size;
-	char *icon_name;
-	GtkIconTheme *theme;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
+	RBRemovableMediaSourcePrivate *priv;
 
-	source = G_OBJECT_CLASS(rb_removable_media_source_parent_class)
-			->constructor (type, n_construct_properties, construct_properties);
+	RB_CHAIN_GOBJECT_METHOD (rb_removable_media_source_parent_class, constructed, object);
+	priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (object);
 
-	g_object_get (source, "volume", &volume, NULL);
-	if (volume != NULL) {
-		drive = gnome_vfs_volume_get_drive (volume);
-		if (drive != NULL) {
-			display_name = gnome_vfs_drive_get_display_name (drive);
-			gnome_vfs_drive_unref (drive);
-		} else {
-			display_name = gnome_vfs_volume_get_display_name (volume);
-		}
-		icon_name = gnome_vfs_volume_get_icon (volume);
-		g_object_unref (volume);
+	/* prefer mount details to volume details, as the nautilus sidebar does */
+	if (priv->mount != NULL) {
+		mount = g_object_ref (priv->mount);
+	} else if (priv->volume != NULL) {
+		mount = g_volume_get_mount (priv->volume);
 	} else {
-		display_name = g_strdup ("Unknown Device");
-		icon_name = g_strdup ("multimedia-player");
+		mount = NULL;
 	}
 
-	g_object_set (source, "name", display_name, NULL);
+	if (mount != NULL) {
+		display_name = g_mount_get_name (mount);
+		icon = g_mount_get_icon (mount);
+		rb_debug ("details from mount: display name = %s, icon = %p", display_name, icon);
+	} else if (priv->volume != NULL) {
+		display_name = g_volume_get_name (priv->volume);
+		icon = g_volume_get_icon (priv->volume);
+		rb_debug ("details from volume: display name = %s, icon = %p", display_name, icon);
+	} else {
+		display_name = g_strdup ("Unknown Device");
+		icon = g_themed_icon_new ("multimedia-player");
+	}
+
+	g_object_set (object, "name", display_name, NULL);
 	g_free (display_name);
 
-	theme = gtk_icon_theme_get_default ();
-	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
-	pixbuf = gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
-	g_free (icon_name);
+	if (icon == NULL) {
+		rb_debug ("no icon set");
+		pixbuf = NULL;
+	} else if (G_IS_THEMED_ICON (icon)) {
+		GtkIconTheme *theme;
+		const char * const *names;
+		gint size;
+		int i;
 
-	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+		theme = gtk_icon_theme_get_default ();
+		gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
+
+		i = 0;
+		names = g_themed_icon_get_names (G_THEMED_ICON (icon));
+		while (names[i] != NULL && pixbuf == NULL) {
+			rb_debug ("looking up themed icon: %s", names[i]);
+			pixbuf = gtk_icon_theme_load_icon (theme, names[i], size, 0, NULL);
+			i++;
+		}
+
+	} else if (G_IS_LOADABLE_ICON (icon)) {
+		rb_debug ("loading of GLoadableIcons is not implemented yet");
+		pixbuf = NULL;
+	}
+
+	rb_source_set_pixbuf (RB_SOURCE (object), pixbuf);
 	if (pixbuf != NULL) {
 		g_object_unref (pixbuf);
 	}
-
-	return source;
+	if (mount != NULL) {
+		g_object_unref (mount);
+	}
+	g_object_unref (icon);
 }
 
 static void
@@ -188,9 +215,13 @@ rb_removable_media_source_dispose (GObject *object)
 	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (object);
 
 	if (priv->volume) {
-		gnome_vfs_volume_unref (priv->volume);
+		g_object_unref (priv->volume);
 		priv->volume = NULL;
 	}
+	if (priv->mount) {
+		g_object_unref (priv->mount);
+		priv->mount = NULL;
+	}
 
 	G_OBJECT_CLASS (rb_removable_media_source_parent_class)->dispose (object);
 }
@@ -206,10 +237,21 @@ rb_removable_media_source_set_property (GObject *object,
 	switch (prop_id) {
 	case PROP_VOLUME:
 		if (priv->volume) {
-			gnome_vfs_volume_unref (priv->volume);
+			g_object_unref (priv->volume);
 		}
 		priv->volume = g_value_get_object (value);
-		gnome_vfs_volume_ref (priv->volume);
+		if (priv->volume) {
+			g_object_ref (priv->volume);
+		}
+		break;
+	case PROP_MOUNT:
+		if (priv->mount) {
+			g_object_unref (priv->mount);
+		}
+		priv->mount = g_value_get_object (value);
+		if (priv->mount) {
+			g_object_ref (priv->mount);
+		}
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -227,8 +269,10 @@ rb_removable_media_source_get_property (GObject *object,
 
 	switch (prop_id) {
 	case PROP_VOLUME:
-		gnome_vfs_volume_ref (priv->volume);
-		g_value_take_object (value, priv->volume);
+		g_value_set_object (value, priv->volume);
+		break;
+	case PROP_MOUNT:
+		g_value_set_object (value, priv->mount);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -256,17 +300,19 @@ impl_delete_thyself (RBSource *source)
 	g_object_unref (db);
 }
 
-#ifdef ENABLE_TRACK_TRANSFER
-
 struct _TrackAddedData {
 	RBRemovableMediaSource *source;
 	char *mimetype;
 };
 
 static void
-_track_added_cb (RhythmDBEntry *entry, const char *uri, gint64 dest_size, struct _TrackAddedData *data)
+_track_added_cb (RhythmDBEntry *entry, const char *uri, GError *error, struct _TrackAddedData *data)
 {
-	rb_removable_media_source_track_added (data->source, entry, uri, dest_size, data->mimetype);
+	if (error == NULL) {
+		rb_removable_media_source_track_added (data->source, entry, uri, data->mimetype);
+	} else {
+		rb_removable_media_source_track_add_error (data->source, entry, uri, error);
+	}
 	g_free (data->mimetype);
 	g_free (data);
 }
@@ -310,7 +356,7 @@ impl_paste (RBSource *source, GList *entries)
 		entry_type = rhythmdb_entry_get_entry_type (entry);
 
 		if (entry_type == our_entry_type ||
-		    entry_type->category != RHYTHMDB_ENTRY_NORMAL) {
+		    !rb_removable_media_source_should_paste (RB_REMOVABLE_MEDIA_SOURCE (source), entry)) {
 			goto impl_paste_end;
 		}
 
@@ -363,7 +409,7 @@ impl_paste (RBSource *source, GList *entries)
 		added_data->mimetype = g_strdup (mimetype);
 		rb_removable_media_manager_queue_transfer (rm_mgr, entry,
 							   dest, mime_types,
-							   (RBTranferCompleteCallback)_track_added_cb, added_data);
+							   (RBTransferCompleteCallback)_track_added_cb, added_data);
 impl_paste_end:
 		g_free (dest);
 		g_free (mimetype);
@@ -379,41 +425,70 @@ impl_paste_end:
 	g_object_unref (encoder);
 }
 
-#endif
-
 static guint
 impl_want_uri (RBSource *source, const char *uri)
 {
-	GnomeVFSVolume *volume;
-	char *activation_uri;
-	int retval, len;
+	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (source);
+	GVolume *volume;
+	GFile *file;
+	char *device_path, *uri_path;
+	int retval;
+	int len;
 
 	retval = 0;
 
 	/* A default version for use with the audio players
 	 * that use mass storage */
-	if (g_str_has_prefix (uri, "file://") == FALSE)
+	file = g_file_new_for_uri (uri);
+	if (g_file_has_uri_scheme (file, "file") == FALSE) {
+		g_object_unref (file);
 		return 0;
+	}
 
-	g_object_get (G_OBJECT (source),
-		      "volume", &volume,
-		      NULL);
-	if (volume == NULL)
+	/* Deal with the mount root being passed, eg. file:///media/IPODNAME */
+	if (priv->mount) {
+		GFile *root;
+
+		root = g_mount_get_root (priv->mount);
+		retval = g_file_equal (root, file) ? 100 : 0;
+		g_object_unref (root);
+		if (retval) {
+			g_object_unref (file);
+			return retval;
+		}
+		volume = g_mount_get_volume (priv->mount);
+	} else if (priv->volume) {
+		volume = g_object_ref (priv->volume);
+	} else {
 		return 0;
+	}
 
-	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
-	if (activation_uri == NULL)
+	if (volume == NULL) {
+		g_object_unref (file);
 		return 0;
+	}
 
-	len = strlen (uri);
-	if (uri[len - 1] == '/') {
-		if (strncmp (uri, activation_uri, len - 1) == 0)
+	/* Deal with the path to the device node being passed */
+	device_path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	g_object_unref (volume);
+	if (device_path == NULL) {
+		g_object_unref (file);
+		return 0;
+	}
+
+	uri_path = g_file_get_path (file);
+	g_object_unref (file);
+	if (uri_path == NULL)
+		return 0;
+	len = strlen (uri_path);
+	if (uri_path[len - 1] == '/') {
+		if (strncmp (uri_path, device_path, len - 1) == 0)
 			retval = 100;
-	} else if (strcmp (uri, activation_uri) == 0)
+	} else if (strcmp (uri_path, device_path) == 0)
 		retval = 100;
 
-	g_free (activation_uri);
-
+	g_free (device_path);
+	g_free (uri_path);
 	return retval;
 }
 
@@ -550,18 +625,84 @@ rb_removable_media_source_get_mime_types (RBRemovableMediaSource *source)
 		return NULL;
 }
 
+gboolean
+rb_removable_media_source_should_paste_no_duplicate (RBRemovableMediaSource *source,
+						     RhythmDBEntry *entry)
+{
+	RhythmDBEntryType entry_type;
+	RhythmDB *db;
+	RBShell *shell;
+	const char *title;
+	const char *album;
+	const char *artist;
+	GtkTreeModel *query_model;
+	GtkTreeIter iter;
+	gboolean no_match;
+	 
+	RBRemovableMediaSourceClass *rms_class = RB_REMOVABLE_MEDIA_SOURCE_CLASS (g_type_class_peek_parent (RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS (source)));
+	/* chain up to parent impl */
+	if (!rms_class->impl_should_paste (source, entry))
+		return FALSE;
+
+	g_object_get (source, "shell", &shell, "entry-type", &entry_type, NULL);
+	g_object_get (shell, "db", &db, NULL);
+	g_object_unref (shell);
+
+	query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty (db));
+	title = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
+	album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
+	artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+	rhythmdb_do_full_query (db, RHYTHMDB_QUERY_RESULTS (query_model),
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_TYPE, entry_type,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_ARTIST, artist,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_ALBUM, album,
+				RHYTHMDB_QUERY_PROP_EQUALS,
+				RHYTHMDB_PROP_TITLE, title,
+				RHYTHMDB_QUERY_END);
+
+	no_match = (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (query_model),
+						 &iter));
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref(query_model);
+	g_object_unref (db);
+	if (no_match == FALSE) {
+		rb_debug ("not adding %s - %s - %s to  removable device since it's already present\n", title, album, artist);
+	}
+	return no_match;
+}
+
+static gboolean
+impl_should_paste (RBRemovableMediaSource *source, RhythmDBEntry *entry)
+{
+	RhythmDBEntryType entry_type = rhythmdb_entry_get_entry_type (entry);
+	gboolean should_paste = (entry_type->category == RHYTHMDB_ENTRY_NORMAL);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	return should_paste;
+}
+
+gboolean
+rb_removable_media_source_should_paste (RBRemovableMediaSource *source,
+					RhythmDBEntry *entry)
+{
+	RBRemovableMediaSourceClass *klass = RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS (source);
+
+	return klass->impl_should_paste (source, entry);
+}
+
 void
 rb_removable_media_source_track_added (RBRemovableMediaSource *source,
 				       RhythmDBEntry *entry,
 				       const char *uri,
-				       guint64 filesize,
 				       const char *mimetype)
 {
 	RBRemovableMediaSourceClass *klass = RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS (source);
 	gboolean add_to_db = TRUE;
 
 	if (klass->impl_track_added)
-		add_to_db = klass->impl_track_added (source, entry, uri, filesize, mimetype);
+		add_to_db = klass->impl_track_added (source, entry, uri, mimetype);
 
 	if (add_to_db) {
 		RhythmDBEntryType entry_type;
@@ -580,3 +721,24 @@ rb_removable_media_source_track_added (RBRemovableMediaSource *source,
 	}
 }
 
+void
+rb_removable_media_source_track_add_error (RBRemovableMediaSource *source,
+					   RhythmDBEntry *entry,
+					   const char *uri,
+					   GError *error)
+{
+	RBRemovableMediaSourceClass *klass = RB_REMOVABLE_MEDIA_SOURCE_GET_CLASS (source);
+	gboolean show_dialog = TRUE;
+
+	if (klass->impl_track_add_error)
+		show_dialog = klass->impl_track_add_error (source, entry, uri, error);
+
+	if (show_dialog) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+			rb_debug ("not displaying 'file exists' error for %s", uri);
+		} else {
+			rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+		}
+	}
+}
+
