@@ -60,8 +60,11 @@ typedef struct
 	gboolean	(*impl_track_added)	(RBRemovableMediaSource *source,
 						 RhythmDBEntry *entry,
 						 const char *uri,
-						 guint64 filesize,
 						 const char *mimetype);
+	gboolean	(*impl_track_add_error) (RBRemovableMediaSource *source,
+						 RhythmDBEntry *entry,
+						 const char *uri,
+						 GError *error);
 	gboolean	(*impl_should_paste)	(RBRemovableMediaSource *source,
 						 RhythmDBEntry *entry);
 } RBRemovableMediaSourceClass;
@@ -78,8 +81,11 @@ char*		rb_removable_media_source_build_dest_uri 	(RBRemovableMediaSource *source
 void		rb_removable_media_source_track_added		(RBRemovableMediaSource *source,
 								 RhythmDBEntry *entry,
 								 const char *uri,
-								 guint64 filesize,
 								 const char *mimetype);
+void		rb_removable_media_source_track_add_error	(RBRemovableMediaSource *source,
+								 RhythmDBEntry *entry,
+								 const char *uri,
+								 GError *error);
 GList *		rb_removable_media_source_get_mime_types	(RBRemovableMediaSource *source);
 gboolean	rb_removable_media_source_should_paste		(RBRemovableMediaSource *source,
 								 RhythmDBEntry *entry);
