@@ -9,7 +9,7 @@
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
  * GStreamer plugins to be used and distributed together with GStreamer
  * and Rhythmbox. This permission is above and beyond the permissions granted
  * by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -105,9 +105,7 @@ enum
 
 static void rb_sourcelist_class_init (RBSourceListClass *klass);
 static void rb_sourcelist_init (RBSourceList *sourcelist);
-static GObject * rb_sourcelist_constructor (GType type,
-					    guint n_construct_properties,
-					    GObjectConstructParam *construct_properties);
+static void rb_sourcelist_constructed (GObject *object);
 static void rb_sourcelist_finalize (GObject *object);
 static void rb_sourcelist_set_property (GObject *object,
 					guint prop_id,
@@ -141,7 +139,7 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 	o_class = (GObjectClass *) class;
 	object_class = (GtkObjectClass *) class;
 
-	o_class->constructor = rb_sourcelist_constructor;
+	o_class->constructed = rb_sourcelist_constructed;
 	o_class->finalize = rb_sourcelist_finalize;
 	o_class->set_property = rb_sourcelist_set_property;
 	o_class->get_property = rb_sourcelist_get_property;
@@ -203,7 +201,7 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSourceListClass, drop_received),
 			      NULL, NULL,
-			      gtk_marshal_VOID__POINTER_POINTER,
+			      rb_marshal_VOID__POINTER_POINTER,
 			      G_TYPE_NONE,
 			      2,
 			      G_TYPE_POINTER, G_TYPE_POINTER);
@@ -249,20 +247,16 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 	g_type_class_add_private (class, sizeof (RBSourceListPrivate));
 }
 
-static GObject *
-rb_sourcelist_constructor (GType type,
-			   guint n_construct_properties,
-			   GObjectConstructParam *construct_properties)
+static void
+rb_sourcelist_constructed (GObject *object)
 {
 	RBSourceList *sourcelist;
 
-	sourcelist = RB_SOURCELIST (G_OBJECT_CLASS (rb_sourcelist_parent_class)
-			->constructor (type, n_construct_properties, construct_properties));
+	RB_CHAIN_GOBJECT_METHOD (rb_sourcelist_parent_class, constructed, object);
+	sourcelist = RB_SOURCELIST (object);
 
 	gtk_container_add (GTK_CONTAINER (sourcelist),
 			   sourcelist->priv->treeview);
-
-	return G_OBJECT (sourcelist);
 }
 
 static void
@@ -301,15 +295,7 @@ rb_sourcelist_cell_set_background (RBSourceList       *list,
 				      NULL);
 		}
 	} else {
-		color = style->text_aa[GTK_STATE_INSENSITIVE];
-
-		color.red = (color.red + (style->white).red) / 2;
-		color.green = (color.green + (style->white).green) / 2;
-		color.blue = (color.blue + (style->white).blue) / 2;
-
-		g_object_set (cell,
-			      "cell-background-gdk", &color,
-			      NULL);
+		/* don't set background for group heading */
 	}
 }
 
@@ -327,7 +313,7 @@ sourcelist_indent_level1_cell_data_func (GtkTreeViewColumn *tree_column,
 	depth = gtk_tree_path_get_depth (path);
 	gtk_tree_path_free (path);
 	g_object_set (cell,
-		      "text", " ",
+		      "text", "    ",
 		      "visible", depth > 1,
 		      NULL);
 }
@@ -346,7 +332,7 @@ sourcelist_indent_level2_cell_data_func (GtkTreeViewColumn *tree_column,
 	depth = gtk_tree_path_get_depth (path);
 	gtk_tree_path_free (path);
 	g_object_set (cell,
-		      "text", " ",
+		      "text", "    ",
 		      "visible", depth > 2,
 		      NULL);
 }
@@ -972,21 +958,21 @@ rb_sourcelist_find_group_iter (RBSourceList  *sourcelist,
 static void
 sourcelist_get_group (RBSourceList  *sourcelist,
 		      RBSourceGroup *group,
-		      GtkTreeIter   *iter,
+		      GtkTreeIter   *iter_group,
 		      gboolean      *created)
 {
 	gboolean found;
 
 	found = rb_sourcelist_find_group_iter (sourcelist,
 					       group,
-					       iter);
+					       iter_group);
 	if (! found) {
 		if (created != NULL) {
 			*created = TRUE;
 		}
 
-		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), iter, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), iter,
+		gtk_tree_store_append (GTK_TREE_STORE (sourcelist->priv->real_model), iter_group, NULL);
+		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), iter_group,
 				    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, NULL,
 				    RB_SOURCELIST_MODEL_COLUMN_NAME, group->display_name,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, NULL,
