@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -45,6 +45,7 @@
 #include "eel-gconf-extensions.h"
 #include "rb-util.h"
 #include "rhythmdb.h"
+#include "rb-player.h"
 
 /**
  * SECTION:rb-header
@@ -79,8 +80,10 @@ static gboolean slider_press_callback (GtkWidget *widget, GdkEventButton *event,
 static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event, RBHeader *header);
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static void slider_changed_callback (GtkWidget *widget, RBHeader *header);
+static gboolean slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *header);
 
-static void rb_header_elapsed_changed_cb (RBShellPlayer *player, guint elapsed, RBHeader *header);
+static void rb_header_elapsed_changed_cb (RBShellPlayer *player, gint64 elapsed, RBHeader *header);
+static void rb_header_extra_metadata_cb (RhythmDB *db, RhythmDBEntry *entry, const char *property_name, const GValue *metadata, RBHeader *header);
 
 struct RBHeaderPrivate
 {
@@ -104,7 +107,7 @@ struct RBHeaderPrivate
 	long latest_set_time;
 	GtkWidget *elapsed;
 
-	guint elapsed_time;
+	gint64 elapsed_time;		/* nanoseconds */
 	long duration;
 	gboolean seekable;
 };
@@ -114,7 +117,6 @@ enum
 	PROP_0,
 	PROP_DB,
 	PROP_SHELL_PLAYER,
-	PROP_ENTRY,
 	PROP_SEEKABLE,
 	PROP_SLIDER_DRAGGING
 };
@@ -124,6 +126,9 @@ enum
 #define ARTIST_MARKUP(xARTIST) g_markup_printf_escaped (" %s <i>%s</i>", _("by"), xARTIST)
 #define STREAM_MARKUP(xSTREAM) g_markup_printf_escaped (" (%s)", xSTREAM)
 
+#define SCROLL_UP_SEEK_OFFSET	5
+#define SCROLL_DOWN_SEEK_OFFSET -5
+
 G_DEFINE_TYPE (RBHeader, rb_header, GTK_TYPE_HBOX)
 
 static void
@@ -149,18 +154,6 @@ rb_header_class_init (RBHeaderClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE));
 
-	/**
-	 * RBHeader:entry:
-	 *
-	 * The #RhythmDBEntry to display
-	 */
-	g_object_class_install_property (object_class,
-					 PROP_ENTRY,
-					 g_param_spec_boxed ("entry",
-							     "RhythmDBEntry",
-							     "RhythmDBEntry pointer",
-							     RHYTHMDB_TYPE_ENTRY,
-							     G_PARAM_READWRITE));
 	/**
 	 * RBHeader:shell-player:
 	 *
@@ -271,9 +264,18 @@ rb_header_init (RBHeader *header)
 				 "value_changed",
 				 G_CALLBACK (slider_changed_callback),
 				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
+				 "scroll_event",
+				 G_CALLBACK (slider_scroll_callback),
+				 header, 0);
 	gtk_scale_set_draw_value (GTK_SCALE (header->priv->scale), FALSE);
 	gtk_widget_set_size_request (header->priv->scale, 150, -1);
 	gtk_box_pack_start (GTK_BOX (header->priv->scaleline), header->priv->scale, TRUE, TRUE, 0);
+
+	/* currently, nothing sets this.  it should be set on track changes. */
+	header->priv->seekable = TRUE;
+
+	rb_header_sync (header);
 }
 
 static void
@@ -291,7 +293,7 @@ rb_header_finalize (GObject *object)
 }
 
 static void
-rb_header_set_playing_entry_internal (RBHeader *header, RhythmDBEntry *entry)
+rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBHeader *header)
 {
 	if (header->priv->entry == entry)
 		return;
@@ -306,6 +308,8 @@ rb_header_set_playing_entry_internal (RBHeader *header, RhythmDBEntry *entry)
 
 	header->priv->adjustment->upper = header->priv->duration;
 	gtk_adjustment_changed (header->priv->adjustment);
+
+	rb_header_sync (header);
 }
 
 static void
@@ -319,16 +323,21 @@ rb_header_set_property (GObject *object,
 	switch (prop_id) {
 	case PROP_DB:
 		header->priv->db = g_value_get_object (value);
-		break;
-	case PROP_ENTRY:
-		rb_header_set_playing_entry_internal (header, g_value_get_boxed (value));
+		g_signal_connect_object (header->priv->db,
+					 "entry-extra-metadata-notify",
+					 G_CALLBACK (rb_header_extra_metadata_cb),
+					 header, 0);
 		break;
 	case PROP_SHELL_PLAYER:
 		header->priv->shell_player = g_value_get_object (value);
-		g_signal_connect (G_OBJECT (header->priv->shell_player),
-				  "elapsed-changed",
-				  (GCallback) rb_header_elapsed_changed_cb,
-				  header);
+		g_signal_connect_object (header->priv->shell_player,
+					 "elapsed-nano-changed",
+					 G_CALLBACK (rb_header_elapsed_changed_cb),
+					 header, 0);
+		g_signal_connect_object (header->priv->shell_player,
+					 "playing-song-changed",
+					 G_CALLBACK (rb_header_playing_song_changed_cb),
+					 header, 0);
 		break;
 	case PROP_SEEKABLE:
 		header->priv->seekable = g_value_get_boolean (value);
@@ -351,9 +360,6 @@ rb_header_get_property (GObject *object,
 	case PROP_DB:
 		g_value_set_object (value, header->priv->db);
 		break;
-	case PROP_ENTRY:
-		g_value_set_boxed (value, header->priv->entry);
-		break;
 	case PROP_SHELL_PLAYER:
 		g_value_set_object (value, header->priv->shell_player);
 		break;
@@ -391,25 +397,6 @@ rb_header_new (RBShellPlayer *shell_player, RhythmDB *db)
 	return header;
 }
 
-/**
- * rb_header_set_playing_entry:
- * @header: the #RBHeader
- * @entry: the new playing #RhythmDBEntry
- * @seekable: whether the new playing entry is seekable
- *
- * Sets a new playing entry to be displayed in the header.
- */
-void
-rb_header_set_playing_entry (RBHeader *header,
-			     RhythmDBEntry *entry,
-			     gboolean seekable)
-{
-	g_object_set (header,
-		      "entry", entry,
-		      "seekable", seekable,
-		      NULL);
-}
-
 static void
 append_and_free (GString *str,
 		 char *text)
@@ -586,8 +573,6 @@ rb_header_set_show_timeline (RBHeader *header,
 void
 rb_header_sync_time (RBHeader *header)
 {
-	guint seconds;
-
 	if (header->priv->shell_player == NULL)
 		return;
 
@@ -596,10 +581,8 @@ rb_header_sync_time (RBHeader *header)
 		return;
 	}
 
-	seconds = header->priv->elapsed_time;
-
 	if (header->priv->duration > 0) {
-		double progress = (double) seconds;
+		double progress = ((double) header->priv->elapsed_time) / RB_PLAYER_SECOND;
 
 		header->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (header->priv->adjustment, progress);
@@ -661,7 +644,7 @@ slider_moved_callback (GtkWidget *widget,
 	}
 
 	progress = gtk_adjustment_get_value (header->priv->adjustment);
-	header->priv->elapsed_time = (guint) (progress+0.5);
+	header->priv->elapsed_time = (gint64) ((progress+0.5) * RB_PLAYER_SECOND);
 
 	rb_header_update_elapsed (header);
 
@@ -730,17 +713,45 @@ slider_changed_callback (GtkWidget *widget,
 	}
 }
 
+static gboolean
+slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *header)
+{
+	gboolean retval = TRUE;
+	gdouble adj = gtk_adjustment_get_value (header->priv->adjustment);
+
+	switch (event->direction) {
+	case GDK_SCROLL_UP:
+		rb_debug ("slider scrolling up");
+		gtk_adjustment_set_value (header->priv->adjustment, adj + SCROLL_UP_SEEK_OFFSET);
+		break;
+
+	case GDK_SCROLL_DOWN:
+		rb_debug ("slider scrolling down");
+		gtk_adjustment_set_value (header->priv->adjustment, adj + SCROLL_DOWN_SEEK_OFFSET);
+		break;
+
+	default:
+		retval = FALSE;
+		break;
+	}
+
+	return retval;
+}
+
 static void
 rb_header_update_elapsed (RBHeader *header)
 {
+	long seconds;
+
 	/* sanity check */
-	if (header->priv->duration > 0 && header->priv->elapsed_time > header->priv->duration)
+	seconds = header->priv->elapsed_time / RB_PLAYER_SECOND;
+	if (header->priv->duration > 0 && seconds > header->priv->duration)
 		return;
 
 	if (header->priv->entry != NULL) {
 		char *elapsed_text;
 
-		elapsed_text = rb_make_elapsed_time_string (header->priv->elapsed_time,
+		elapsed_text = rb_make_elapsed_time_string (seconds,
 							    header->priv->duration,
 							    !eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY));
 		gtk_label_set_text (GTK_LABEL (header->priv->elapsed), elapsed_text);
@@ -753,9 +764,26 @@ rb_header_update_elapsed (RBHeader *header)
 
 static void
 rb_header_elapsed_changed_cb (RBShellPlayer *player,
-			      guint elapsed,
+			      gint64 elapsed,
 			      RBHeader *header)
 {
 	header->priv->elapsed_time = elapsed;
 	rb_header_sync_time (header);
 }
+
+static void
+rb_header_extra_metadata_cb (RhythmDB *db,
+			     RhythmDBEntry *entry,
+			     const char *property_name,
+			     const GValue *metadata,
+			     RBHeader *header)
+{
+	if (entry != header->priv->entry)
+		return;
+
+	if (g_str_equal (property_name, RHYTHMDB_PROP_STREAM_SONG_TITLE) ||
+	    g_str_equal (property_name, RHYTHMDB_PROP_STREAM_SONG_ARTIST) ||
+	    g_str_equal (property_name, RHYTHMDB_PROP_STREAM_SONG_ALBUM)) {
+		rb_header_sync (header);
+	}
+}
