@@ -4,13 +4,21 @@
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
- *  Copyright (C) 2006 James Livingston <jrl@ids.org.au>
+ *  Copyright (C) 2006 James Livingston <doclivingston@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -41,8 +49,8 @@
 static void rb_library_browser_class_init (RBLibraryBrowserClass *klass);
 static void rb_library_browser_init (RBLibraryBrowser *entry);
 static void rb_library_browser_finalize (GObject *object);
-static GObject* rb_library_browser_constructor (GType type, guint n_construct_properties,
-						GObjectConstructParam *construct_properties);
+static void rb_library_browser_dispose (GObject *object);
+static void rb_library_browser_constructed (GObject *object);
 static void rb_library_browser_set_property (GObject *object,
 					     guint prop_id,
 					     const GValue *value,
@@ -64,9 +72,41 @@ static void rb_library_browser_views_changed (GConfClient *client,
 					      GConfEntry *entry,
 					      RBLibraryBrowser *widget);
 
+typedef struct _RBLibraryBrowserRebuildData RBLibraryBrowserRebuildData;
+
+static void destroy_idle_rebuild_model (RBLibraryBrowserRebuildData *data);
+
 G_DEFINE_TYPE (RBLibraryBrowser, rb_library_browser, GTK_TYPE_HBOX)
 #define RB_LIBRARY_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_BROWSER, RBLibraryBrowserPrivate))
 
+/**
+ * SECTION:rb-library-browser
+ * @short_description: album/artist/genre browser widget
+ * @include: rb-library-browser.h
+ *
+ * This widget contains a set of #RBPropertyView<!-- -->s backed by
+ * #RhythmDBPropertyModel<!-- -->s and constructs a chain of
+ * #RhythmDBQueryModel<!-- -->s to perform filtering of the entries
+ * in a source.
+ *
+ * It operates on an input query model, containing the full set of
+ * entries that may be displayed in the source, and produces an
+ * output query model containing those entries that match the current
+ * selection.
+ *
+ * When the selection in any of the property views changes, or when
+ * #rb_library_browser_reset or #rb_library_browser_set_selection are
+ * called to manipulate the selection, the query chain is rebuilt
+ * asynchronously to update the property views.
+ */
+
+struct _RBLibraryBrowserRebuildData
+{
+	RBLibraryBrowser *widget;
+	int rebuild_prop_index;
+	int rebuild_idle_id;
+};
+
 typedef struct
 {
 	RhythmDB *db;
@@ -79,6 +119,8 @@ typedef struct
 
 	GHashTable *property_views;
 	GHashTable *selections;
+
+	RBLibraryBrowserRebuildData *rebuild_data;
 } RBLibraryBrowserPrivate;
 
 enum
@@ -108,10 +150,16 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = rb_library_browser_finalize;
-	object_class->constructor = rb_library_browser_constructor;
+	object_class->dispose = rb_library_browser_dispose;
+	object_class->constructed = rb_library_browser_constructed;
 	object_class->set_property = rb_library_browser_set_property;
 	object_class->get_property = rb_library_browser_get_property;
 
+	/**
+	 * RBLibraryBrowser:db:
+	 *
+	 * #RhythmDB instance
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -119,6 +167,14 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 							      "RhythmDB instance",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBLibraryBrowser:input-model:
+	 *
+	 * This #RhythmDBQueryModel defines the set of entries that
+	 * the browser filters.  This property is not writeable.
+	 * To set a new input query model, use 
+	 * #rb_library_browser_set_model.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_INPUT_MODEL,
 					 g_param_spec_object ("input-model",
@@ -126,6 +182,18 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 							      "input RhythmDBQueryModel instance",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READABLE));
+	/**
+	 * RBLibraryBrowser:output-model:
+	 *
+	 * This #RhythmDBQueryModel contains the filtered set of
+	 * entries.  It is a subset of the entries contained in the
+	 * input model.  This should be used as the model backing
+	 * the source's entry view.
+	 *
+	 * Sources using this widget should connect to the notify
+	 * signal for this property, updating their entry view when
+	 * it changes.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_OUTPUT_MODEL,
 					 g_param_spec_object ("output-model",
@@ -133,6 +201,11 @@ rb_library_browser_class_init (RBLibraryBrowserClass *klass)
 							      "output RhythmDBQueryModel instance",
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READABLE));
+	/**
+	 * RBLibraryBrowser:entry-type:
+	 *
+	 * The type of entries to use in the browser.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
 					 g_param_spec_boxed ("entry-type",
@@ -155,20 +228,16 @@ rb_library_browser_init (RBLibraryBrowser *widget)
 	priv->selections = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)rb_list_deep_free);
 }
 
-static GObject *
-rb_library_browser_constructor (GType type,
-				guint n_construct_properties,
-				GObjectConstructParam *construct_properties)
+static void
+rb_library_browser_constructed (GObject *object)
 {
-	RBLibraryBrowserClass *klass;
 	RBLibraryBrowser *browser;
 	RBLibraryBrowserPrivate *priv;
 	int i;
 
-	klass = RB_LIBRARY_BROWSER_CLASS (g_type_class_peek (RB_TYPE_LIBRARY_BROWSER));
+	RB_CHAIN_GOBJECT_METHOD (rb_library_browser_parent_class, constructed, object);
 
-	browser = RB_LIBRARY_BROWSER (G_OBJECT_CLASS (rb_library_browser_parent_class)->
-			constructor (type, n_construct_properties, construct_properties));
+	browser = RB_LIBRARY_BROWSER (object);
 	priv = RB_LIBRARY_BROWSER_GET_PRIVATE (browser);
 
 	for (i = 0; i < num_browser_properties; i++) {
@@ -190,33 +259,57 @@ rb_library_browser_constructor (GType type,
 					 browser, 0);
 		gtk_widget_show_all (GTK_WIDGET (view));
 		gtk_widget_set_no_show_all (GTK_WIDGET (view), TRUE);
-		gtk_box_pack_start_defaults (GTK_BOX (browser), GTK_WIDGET (view));
+		gtk_box_pack_start (GTK_BOX (browser), GTK_WIDGET (view), TRUE, TRUE, 0);
 	}
 
 	update_browser_views_visibility (browser);
 	priv->browser_view_notify_id =
 		eel_gconf_notification_add (CONF_UI_BROWSER_VIEWS,
 				(GConfClientNotifyFunc) rb_library_browser_views_changed, browser);
-
-	return G_OBJECT (browser);
 }
+
+static void
+rb_library_browser_dispose (GObject *object)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
+	
+	if (priv->browser_view_notify_id != 0) {
+		eel_gconf_notification_remove (priv->browser_view_notify_id);
+		priv->browser_view_notify_id = 0;
+	}
+
+	if (priv->rebuild_data != NULL) {
+		/* this looks a bit odd, but removing the idle handler cleans up the
+		 * data too.
+		 */
+		guint id = priv->rebuild_data->rebuild_idle_id;
+		priv->rebuild_data = NULL;
+		g_source_remove (id);
+	}
+	
+	if (priv->db != NULL) {
+		g_object_unref (priv->db);
+		priv->db = NULL;
+	}
+
+	if (priv->input_model != NULL) {
+		g_object_unref (priv->input_model);
+		priv->input_model = NULL;
+	}
+
+	if (priv->output_model != NULL) {
+		g_object_unref (priv->output_model);
+		priv->output_model = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_library_browser_parent_class)->dispose (object);
+}
+
 static void
 rb_library_browser_finalize (GObject *object)
 {
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (object);
 
-	eel_gconf_notification_remove (priv->browser_view_notify_id);
-
-	if (priv->db != NULL) {
-		g_object_unref (priv->db);
-	}
-	if (priv->input_model != NULL) {
-		g_object_unref (priv->input_model);
-	}
-	if (priv->output_model != NULL) {
-		g_object_unref (priv->output_model);
-	}
-
 	g_hash_table_destroy (priv->property_views);
 	g_hash_table_destroy (priv->selections);
 
@@ -278,6 +371,13 @@ rb_library_browser_get_property (GObject *object,
 	}
 }
 
+/**
+ * rb_library_browser_new:
+ * @db: the #RhythmDB instance
+ * @entry_type: the entry type to use in the browser
+ *
+ * Return value: a new RBLibraryBrowser
+ */
 RBLibraryBrowser *
 rb_library_browser_new (RhythmDB *db,
 			RhythmDBEntryType entry_type)
@@ -369,6 +469,14 @@ reset_view_cb (RhythmDBPropType prop,
 	rb_property_view_set_selection (view, NULL);
 }
 
+/**
+ * rb_library_browser_reset:
+ * @widget: a #RBLibraryBrowser
+ *
+ * Clears all selections in the browser.
+ *
+ * Return value: TRUE if anything was changed
+ */
 gboolean
 rb_library_browser_reset (RBLibraryBrowser *widget)
 {
@@ -399,6 +507,14 @@ construct_query_cb (RhythmDBPropType type,
 					     selections);
 }
 
+/**
+ * rb_library_browser_construct_query:
+ * @widget: a #RBLibraryBrowser
+ *
+ * Constructs a #RhythmDBQuery from the current selections in the browser.
+ *
+ * Return value: a #RhythmDBQuery constructed from the current selection.
+ */
 RhythmDBQuery *
 rb_library_browser_construct_query (RBLibraryBrowser *widget)
 {
@@ -418,6 +534,12 @@ rb_library_browser_construct_query (RBLibraryBrowser *widget)
 	return query;
 }
 
+/**
+ * rb_library_browser_has_selection:
+ * @widget: a #RBLibraryBrowser
+ *
+ * Return value: TRUE if any items in the browser are selected.
+ */
 gboolean
 rb_library_browser_has_selection (RBLibraryBrowser *widget)
 {
@@ -597,6 +719,42 @@ rebuild_child_model (RBLibraryBrowser *widget,
 	g_object_unref (base_model);
 }
 
+static gboolean
+idle_rebuild_model (RBLibraryBrowserRebuildData *data)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (data->widget);
+
+	priv->rebuild_data = NULL;
+	rebuild_child_model (data->widget, data->rebuild_prop_index, FALSE);
+	return FALSE;
+}
+
+static void
+destroy_idle_rebuild_model (RBLibraryBrowserRebuildData *data)
+{
+	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (data->widget);
+	RBPropertyView *view;
+	RhythmDBPropType prop_type;
+	
+	prop_type = browser_properties[data->rebuild_prop_index].type;
+	view = g_hash_table_lookup (priv->property_views, (gpointer)prop_type);
+	if (view != NULL) {
+		ignore_selection_changes (data->widget, view, FALSE);
+	}
+
+	priv->rebuild_data = NULL;
+	g_object_unref (data->widget);
+	g_free (data);
+}
+
+/**
+ * rb_library_browser_set_selection:
+ * @widget: a #RBLibraryBrowser
+ * @type: the property for which to set the selection
+ * @selection: a list of strings to select
+ *
+ * Replaces any current selection for the specified property.
+ */
 void
 rb_library_browser_set_selection (RBLibraryBrowser *widget,
 				  RhythmDBPropType type,
@@ -605,6 +763,8 @@ rb_library_browser_set_selection (RBLibraryBrowser *widget,
 	RBLibraryBrowserPrivate *priv = RB_LIBRARY_BROWSER_GET_PRIVATE (widget);
 	GList *old_selection;
 	RBPropertyView *view;
+	int rebuild_index;
+	RBLibraryBrowserRebuildData *rebuild_data;
 
 	old_selection = g_hash_table_lookup (priv->selections, (gpointer)type);
 
@@ -616,15 +776,42 @@ rb_library_browser_set_selection (RBLibraryBrowser *widget,
 	else
 		g_hash_table_remove (priv->selections, (gpointer)type);
 
-	view = g_hash_table_lookup (priv->property_views, (gpointer)type);
-	if (view)
-		ignore_selection_changes (widget, view, TRUE);
+	rebuild_index = prop_to_index (type);
+	if (priv->rebuild_data != NULL) {
+		rebuild_data = priv->rebuild_data;
+		if (rebuild_data->rebuild_prop_index <= rebuild_index) {
+			/* already rebuilding a model further up the chain,
+			 * so we don't need to do anything for this one.
+			 */
+			return;
+		}
+		g_source_remove (rebuild_data->rebuild_idle_id);
+		rebuild_data = NULL;
+	}
 
-	rebuild_child_model (widget, prop_to_index (type), FALSE);
-	if (view)
-		ignore_selection_changes (widget, view, FALSE);
+	view = g_hash_table_lookup (priv->property_views, (gpointer)type);
+	if (view) {
+		ignore_selection_changes (widget, view, TRUE);
+	}
+
+	rebuild_data = g_new0 (RBLibraryBrowserRebuildData, 1);
+	rebuild_data->widget = g_object_ref (widget);
+	rebuild_data->rebuild_prop_index = rebuild_index;
+	rebuild_data->rebuild_idle_id =
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+				 (GSourceFunc) idle_rebuild_model,
+				 rebuild_data,
+				 (GDestroyNotify) destroy_idle_rebuild_model);
+	priv->rebuild_data = rebuild_data;
 }
 
+/**
+ * rb_library_browser_get_property_views:
+ * @widget: a #RBLibraryBrowser
+ *
+ * Return value: a GList containing the #RBPropertyView<!-- -->s
+ *  in the browser.
+ */
 GList*
 rb_library_browser_get_property_views (RBLibraryBrowser *widget)
 {
@@ -633,6 +820,14 @@ rb_library_browser_get_property_views (RBLibraryBrowser *widget)
 	return rb_collate_hash_table_values (priv->property_views);
 }
 
+/**
+ * rb_library_browser_get_property_view:
+ * @widget: a #RBLibraryBrowser
+ * @type: the property
+ *
+ * Return value: the #RBPropertyView for the specified property, or
+ *  NULL if there isn't one
+ */
 RBPropertyView *
 rb_library_browser_get_property_view (RBLibraryBrowser *widget,
 				      RhythmDBPropType type)
@@ -644,6 +839,18 @@ rb_library_browser_get_property_view (RBLibraryBrowser *widget,
 	return view;
 }
 
+/**
+ * rb_library_browser_set_model:
+ * @widget: a #RBLibraryBrowser
+ * @model: the new input #RhythmDBQueryModel
+ * @query_pending: if TRUE, the caller promises to run a
+ *  query to populate the input query model.
+ *
+ * Specifies a new input query model for the browser.
+ * This should be the query model constructed from the 
+ * current search text, or the basic query model for the 
+ * source if there is no search text.
+ */
 void
 rb_library_browser_set_model (RBLibraryBrowser *widget,
 			      RhythmDBQueryModel *model,
